package com.SgwMiniSetup.common;

import java.awt.Desktop;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.SgwMiniSetup.sample.SgwMiniSetup;
import com.SgwMiniSetup.thread.SgwSearchThread;
import com.SgwSetting.common.CommonParam;
import com.SgwSetting.thread.SearchSgwThread;
import com.SgwSetting.thread.SearchSgwThread0;
import com.sun.glass.ui.TouchInputSupport;

public class CommonFunc {

	/**
	 * プロパティファイルを読み込む
	 *
	 * @return
	 */
	public static void loadProperties() {
		 // Propertiesオブジェクトを生成
	      Properties properties = new Properties();

	      try {
	    	  // ファイルを読み込む
	          properties.load(new FileInputStream("conf/conf.properties"));

	          // サーバホスト登録
	          String prop = properties.getProperty("RegistServerHost", "false");
	          if (prop != null && prop.length() > 0) {
	        	  if (prop.toLowerCase().equals("true")) {
		        	  com.SgwMiniSetup.common.CommonParam.registServerHost = true;
		          } else {
		        	  com.SgwMiniSetup.common.CommonParam.registServerHost = false;
		          }
	          }

	          // サーバホストIP
	          prop = properties.getProperty("ServerHostName", "");
	          if (prop != null && prop.length() > 0) {
	        	  com.SgwMiniSetup.common.CommonParam.serverHostName = prop;
	          }

	          // サーバ括登録
	          prop = properties.getProperty("RegistAllServerHost", "false");
	          if (prop != null && prop.length() > 0) {
	        	  if (prop.toLowerCase().equals("true")) {
		        	  com.SgwMiniSetup.common.CommonParam.registAllServerHost = true;
		          } else {
		        	  com.SgwMiniSetup.common.CommonParam.registAllServerHost = false;
		          }
	          }
	          
	          // 最大SGW検索時間
	          prop = properties.getProperty("MaxSgwSearchingTime", "60");
	          if (prop != null && prop.length() > 0) {
	        	  if (isNumber(prop)) {
	        		  try {
	        			  com.SgwMiniSetup.common.CommonParam.maxSgwSearchingTime = Integer.parseInt(prop);
	        		  } catch(Exception e) {}
	        		  
	        		  //最大検索時間を再設定
	        		  com.SgwSetting.common.CommonParam.MAX_WAIT_TIME = com.SgwMiniSetup.common.CommonParam.maxSgwSearchingTime;
		          }
	          }

	       } catch (FileNotFoundException e) {
	          // TODO Auto-generated catch block
	          System.out.println(e.getMessage());
	       } catch (IOException e) {
	          // TODO Auto-generated catch block
	          e.printStackTrace();
	       }
	}
	
	/**
	 * 数値判定
	 *
	 * @param String num 文字列を指定
	 * @return boolean 数値かどうかを返す
	 */
	public static boolean isNumber(String num) {
	    String regex = "^\\-?[0-9]*\\.?[0-9]+$";
	    Pattern p = Pattern.compile(regex);
	    Matcher m = p.matcher(num);
	    return m.find();
	}

	/**
	 * 指定したテキストファイルの内容を読み込んで返す
	 *
	 * @param String file_path 読み込むテキストファイルのパス
	 * @return String contents 読み込んだテキストファイルの内容
	 * @throws URISyntaxException
	 */
	public static String readFile(String file_path) throws URISyntaxException {
		StringBuilder buf = new StringBuilder();
		BufferedReader br = null;

		// ファイルが存在しない場合は空を返す
		if (!(new File(file_path)).exists()) {
			return "";
		}

		try {
			// 改行コード取得
			String crlf = "\n";

			// ファイル読み込み
			File file = new File(file_path);
			//br = new BufferedReader(new FileReader(file));
			//文字コードを指定する（※指定しないと、JARファイルにしたときに、文字コードエラー発生する）
			br = new BufferedReader(new InputStreamReader((new FileInputStream(file)), "UTF-8"));
			String line;

			// ファイル内容を1行ずつ取得し、改行コードを加えてバッファ出力
			while((line = br.readLine()) != null) {
				//buf.append(line + crlf); //【＠Ver.2.04】：改行コードは不要（理由：1行のみしか読み込まないし、不要なキャラクタを含めない為）
				buf.append(line);
			}

		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (br != null) {
					br.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		// バッファの内容を取得
		String contents = buf.toString();

		return contents;
	}

	/**
	 * 指定したテキストファイルに内容を書き込む
	 *
	 * @param String file_path 書き込むテキストファイルのパス
	 * @return String contents 書き込むテキストファイルの内容
	 * @throws URISyntaxException
	 */
	public static void writeFile(String file_path, String contents) throws URISyntaxException {


		// ファイルに見つかったSGWのIPアドレスを書き込み(上書き)
		try {
			// ファイル読み込み
			//URL fileUrl = this.getClass().getClassLoader().getResource(file_path);
			//File file = new File(fileUrl.toURI());
			File file = new File(file_path);
			file.createNewFile();

			if(file.exists() && file.isFile() && file.canWrite()) {
				//PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
				//文字コードを指定する（※指定しないと、JARファイルにしたときに、文字コードエラー発生する）
				PrintWriter pw = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file),"UTF-8")));
				pw.write(contents);
				pw.close();
			} else {
				throw new IOException("ファイルに書き込めません");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * スマートゲートウェイの登録状態を取得する。【＠Ver.2.04】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイの登録状態を戻します。
	 */
	public static String getSgwRegistrationStatus(String resStr) {
		String ret = null;
		if (resStr != null) {
			int index = resStr.indexOf(CommonParam.SGW_REGIST_STATUS_LBL);
			int index_en = resStr.indexOf(CommonParam.SGW_REGIST_STATUS_LBL_EN);
			// 日本語版SGWの処理
			if (index > 0) {
				int firstIndex = index + CommonParam.SGW_REGIST_STATUS_LBL.length();
				int lastIndex = resStr.indexOf("<", firstIndex);
				// 【2017/1/24　revised】： SGW-Plus Ver2.01対応
				if (resStr.indexOf(CommonParam.SGW_CONNECTION_STATUS_LBL, firstIndex) > 0){
					lastIndex = resStr.indexOf(CommonParam.SGW_CONNECTION_STATUS_LBL, firstIndex) - 3;
				}
				ret = resStr.substring(firstIndex, lastIndex);	// 文字取得
				ret = ret.trim();	// 空欄を削除
			// 英語版SGWの処理
			} else if (index_en > 0) {
				int firstIndex = index_en + CommonParam.SGW_REGIST_STATUS_LBL_EN.length();
				int lastIndex = resStr.indexOf("<", firstIndex);
				ret = resStr.substring(firstIndex, lastIndex);	// 文字取得
				ret = ret.trim();	// 空欄を削除
			}
		}
		return ret;
	}

	/**
	 * スマートゲートウェイのタイプをHTMLレスポンスから取得する。【＠Ver.2.04】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプを戻します（デフォールトは、SgwMiniSetup.TYPE_SGW_MINI）
	 */
	public static int getSgwTypeByResString(String resStr) {
		int checkSgw = checkRespBodyForSGW(resStr);
		int checkSgwMini = checkRespBodyForSGWmini(resStr);
		int checkSgwPlus = checkRespBodyForSGWplus(resStr);
		int checkSgwPlusV2 = checkRespBodyForSGWplusV2(resStr);
		int checkSgwPlusV3 = checkRespBodyForSGWplusV3(resStr);
		int checkSgwPlusMC = checkRespBodyForSGWplusMC(resStr);
		int checkSgwPlusV4 = checkRespBodyForSGWplusV4(resStr);
		
		// SGWタイプを判定する
		if (checkSgw >= 0 && checkSgwMini < 0 && checkSgwPlus < 0) {
			return SgwMiniSetup.TYPE_SGW;
		} else if (checkSgw >= 0 && checkSgwPlus >= 0) {
			if (checkSgwPlusV2 > 0 && checkSgwPlusV4 < 0) {
				return SgwMiniSetup.TYPE_SGW_PLUS_V2;
			} else if (checkSgwPlusV3 > 0) {
				return SgwMiniSetup.TYPE_SGW_PLUS_V3;
			} else if (checkSgwPlusMC > 0) {
				return SgwMiniSetup.TYPE_SGW_PLUS_MC;
			} else if (checkSgwPlusV4 > 0) {
				return SgwMiniSetup.TYPE_SGW_PLUS_V4;
			} else {
				return SgwMiniSetup.TYPE_SGW_PLUS;
			}
		} else if (checkSgwPlusMC > 0) {
			return SgwMiniSetup.TYPE_SGW_PLUS_MC;
		} else if (checkSgw >= 0) {
			return SgwMiniSetup.TYPE_SGW_MINI;
		}
		return -1;
	}

	/**
	 * スマートゲートウェイのタイプ判定（for SGW）　【＠Ver.2.04】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGW(String resStr)
	{
		int res_for_sgw = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW);
		return res_for_sgw;
	}

	/**
	 * スマートゲートウェイのタイプ判定（for SGW-mini）　【＠Ver.2.04】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGWmini(String resStr)
	{
		int res_for_sgw_mini = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_MINI);
		return res_for_sgw_mini;
	}

	/**
	 * スマートゲートウェイのタイプ判定（for SGW-Plus）　【＠Ver.2.04】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGWplus(String resStr)
	{
		int res_for_sgw_plus = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_PLUS);
		return res_for_sgw_plus;
	}

	/**
	 * スマートゲートウェイのタイプ判定（for SGW-Plus Ver.2.xx）　【＠Ver.2.17】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGWplusV2(String resStr)
	{
		int res_for_sgw_plus = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_PLUS_V2);
		return res_for_sgw_plus;
	}
	
	/**
	 * スマートゲートウェイのタイプ判定（for SGW-Plus Ver.3.xx）　【＠Ver.2.22】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGWplusV3(String resStr)
	{
		int res_for_sgw_plus = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_PLUS_V3);
		return res_for_sgw_plus;
	}
	
	/**
	 * スマートゲートウェイのタイプ判定（for SGW-Plus Ver.3.xx）　【＠Ver.2.22】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGWplusMC(String resStr)
	{
		int res_for_sgw_plus = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_PLUS_MC);
		return res_for_sgw_plus;
	}
	
	/**
	 * スマートゲートウェイのタイプ判定（for SGW-Plus Ver.4.xx）　【＠Ver.2.27】
	 *
	 * @param resStr HTMLレスポンス
	 * @return スマートゲートウェイのタイプ判定結果を戻します。
	 */
	private static int checkRespBodyForSGWplusV4(String resStr)
	{
		int res_for_sgw_plus = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_PLUS_V4);
		if (res_for_sgw_plus < 0) {
			//int res_for_sgw_plus_v2 = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_SGW_PLUS_V2);
			//if (res_for_sgw_plus_v2 > 0) {
				res_for_sgw_plus = resStr.indexOf(CommonParam.SEARCH_STR_HTMLFILE_FOR_TWS_7M);
			//}
		}
		return res_for_sgw_plus;
	}

	/**
	 * Javaのバージョンをチェックする
	 *
	 *　@param num
	 * @return Javaのバージョンがサポート範囲内かどうかを戻します。
	 */
	public static boolean checkSupportedJavaVersion() {
		String java_version = System.getProperty("java.version");
		try {
			double d_version = CommonFunc.parseDouble(java_version.substring(0, 3));
			// Javaのバージョンが1.8.0以前だったら
			if (d_version < 1.8d) {
				return false;
			}
		} catch (Exception e) {}
		return true;
	}
	
	/**
	 * Oracle Javaのバージョンをチェックする
	 *
	 *　@param num
	 * @return Javaのバージョンがサポート範囲内かどうかを戻します。
	 */
	public static boolean checkOpenJava() {
		String java_vendor = System.getProperty("java.vendor");
		String java_vendor_url = System.getProperty("java.vendor.url");
		String jvn_name = System.getProperty("java.vm.name");
		//System.out.println("[check JVM]: java.vm.name = [" + System.getProperty("java.vm.name") + "]");
		
		try {
			// Oracle Java（ライセンス版）だったら
			if ((java_vendor != null && java_vendor.toLowerCase().contains("oracle"))
					&& (java_vendor_url != null && java_vendor_url.toLowerCase().contains("oracle"))
					&& (jvn_name != null && jvn_name.contains("Java HotSpot(TM)"))) {
				return false;
			}
		} catch (Exception e) {}
		return true;
	}

	/**
	 * 文字列をDouble値に変換する
	 *
	 *　@param num
	 * @return Double値かどうかを戻します。
	 */
	public static double parseDouble(String num) {
		if (num == null) return 0.0d;
		try {
			String tmpNum = num;
			if (tmpNum.indexOf(",") >= 0) {
				tmpNum = tmpNum.replace(",", "");	// カンマを取り除く
			}
			double n = Double.parseDouble(tmpNum);
			return n;
		} catch (NumberFormatException e) {
			return 0.0d;
		} catch (Exception e) {
			e.printStackTrace();
			return 0.0d;
		}
	}
	
	/*
	 * 検索用スレッドのリストを扱うためのメソッド群
	 */
	public static List<Thread> getSearchThreadList() {
		createSearchThreadList();
		return CommonParam.listSearchThread;
	}
	
	private static void createSearchThreadList() {
		if (CommonParam.listSearchThread == null) {
			CommonParam.listSearchThread = new ArrayList<Thread>();
		}
	}
	
	public static void addSearchThreadList(Thread thread) {
		if (thread != null 
				&& CommonParam.listSearchThread != null
				&& !CommonParam.listSearchThread.contains(thread)) {
			try {
				CommonParam.listSearchThread.add(thread);
			} catch (Exception e) {}
		}
	}
	
	public static boolean removeSearchThreadFromList(Thread thread) {
		if (thread != null 
				&& CommonParam.listSearchThread != null
				&& CommonParam.listSearchThread.contains(thread)) {
			try {
				CommonParam.listSearchThread.remove(thread);
			} catch (Exception e) {
				return false;
			}
		}
		return true;
	}
	public static boolean clearSearchThreadList() {
		if (CommonParam.listSearchThread != null
				&& !CommonParam.listSearchThread.isEmpty()) {
			try {
				stopAllSearchThreadList();
				CommonParam.listSearchThread.clear();
			} catch (Exception e) {
				return false;
			}
		}
		// リストがなければ新たにリストを生成する
		createSearchThreadList();
		return true;
	}
	
	public static boolean stopAllSearchThreadList() {
		try {
			if (CommonParam.listSearchThread != null) {
				for (Thread th : CommonParam.listSearchThread) {
					if (th != null && th.isAlive()) {
						th.interrupt();
						//((SgwSearchThread) th).stopThread();
					}
				}
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	
	public static Thread getLastSgwSearchThread() {
		if (CommonParam.listSearchThread != null
				&& !CommonParam.listSearchThread.isEmpty()) {
			int index = CommonParam.listSearchThread.size() - 1;
			return CommonParam.listSearchThread.get(index);
		}
		return null;
	}
	
    /**
     * 現在日時を取得する
     */
    public static String getFormatedDate() {
        Calendar cal = Calendar.getInstance();
        //フォーマットパターンを指定して表示する
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(cal.getTime());
    }
    
    /**
     * ログファイルを作成する
     */
    public static File getLogFile() {
    	String filePath = new File(".").getAbsoluteFile().getParent();
    	
    	Calendar cal = Calendar.getInstance();
        //フォーマットパターンを指定して表示する
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String dateStamp = sdf.format(cal.getTime());
    	
        // ログファイルの作成
    	String fileName = filePath + "/log/" + dateStamp + ".csv";
    	File rtnFile = new File(fileName);
    	
    	// フォルダの確認
    	File parFile = rtnFile.getParentFile();
    	if (!parFile.exists() || !parFile.isDirectory()) {
    		try {
    			parFile.mkdirs();	// フォルダを作成する
    		} catch (Exception e) { return null; }
    	}
    	
    	// ファイルの作成
    	if (!rtnFile.exists()) {
    		try {
    			rtnFile.createNewFile();	// ファイルを作成する
    		} catch (Exception e) { return null; }
    	}
    	return rtnFile;
    }
  
    /**
     * ログファイルに書き込む
     */
	public static File writeLogFile(List<String> strArray) {
		try {
			File file = getLogFile();
			
			if (file.isFile() && file.canWrite()) {
				BufferedWriter bw = new BufferedWriter(new FileWriter(file, true));

				for (String str : strArray) {
					bw.write(str);
					bw.newLine();
				}
				bw.close();
				
				return file;
				
			} else {
				System.out.println("ファイルに書き込めません");
			}
		} catch (IOException e) {
			System.out.println(e);
		} 
		
		return null;
	}
	
    /**
     * ノートパッドでファイルを開く
     */
	public static void launchNotePad(File file) {
		Runtime r = Runtime.getRuntime();
		try {
			r.exec("notePad " + file.getAbsolutePath());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 関連付けられたエディタアプリケーションを起動して、ファイルを編集用に開きます。
	 *
	 *　@param url
	 * @return なし
	 */
	public static void launchTextEditorWithDesktop(File file) {
		Desktop desktop = Desktop.getDesktop();
		try {
			if (Desktop.isDesktopSupported()) {
				desktop.open(file);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Mac OSかどうかを判定する
	 *
	 * @return Mac OSかどうかを戻します。
	 */
	public static boolean isMacOS() {
		boolean rtnBoo = false;
		String osname = System.getProperty("os.name");
		// MacOSであったときの処理
		if (osname.indexOf("Mac") >= 0) {
			rtnBoo = true;
		}
		return rtnBoo;
	}
	
	/**
	 * Linux OSかどうかを判定する
	 *
	 * @return Linux OSかどうかを戻します。
	 */
	public static boolean isLinuxOS() {
		boolean rtnBoo = false;
		String osname = System.getProperty("os.name");
		// LinuxOSであったときの処理
		if (osname.indexOf("Linux") >= 0) {
			rtnBoo = true;
		}
		return rtnBoo;
	}
	
	/**
	 * InputStreamをStringに変換する
	 * 
	 * @param is
	 * @param encode
	 * @return 変換された文字列を返す。
	 */
	public static String convertInputStreamToString(InputStream is, String encode) {
		try {
		    InputStreamReader reader = new InputStreamReader(is, encode);
		    StringBuilder builder = new StringBuilder();
		    char[] buffer = new char[512];
		    int read;
		    while (0 <= (read = reader.read(buffer))) {
		        builder.append(buffer, 0, read);
		    }
		    return builder.toString();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
}
