package com.SgwMiniSetup.sample;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

import com.SgwMiniSetup.thread.SgwSearchThread;

/**
 * スマートゲートウェイ 簡単セットアップGUI(PC版) - 独自のダイアログ出力
 *
 * @author Copyright (c) 2013 Enegate Co, ltd.
 * @version 1.00
 */
public class CustomDialog extends JDialog implements ActionListener, WindowListener {
	/**
	 * アプリの定数
	 */
	private static final long serialVersionUID = 1L;

	// ========================================================================================================
	// 画像リソース
	// ========================================================================================================
	private final static String IMG_ICON_LOADING = "res/img/loading.gif"; //
	private final static String IMG_ICON_RESULT1 = "res/img/icon_result1.png"; //
	private final static String IMG_ICON_RESULT2 = "res/img/icon_result2.png"; //

	private final static String IMG_BUTTON_CONFIRM_ON = "res/img/btn_confirm_on.png"; // 確認ボタン
	private final static String IMG_BUTTON_CONFIRM_OFF = "res/img/btn_confirm_off.png"; // 確認ボタン
	
	private JFrame mainFrame = null;

	/**
	 * コンストラクタ
	 *
	 * @param JFrame frame
	 * @param CardLayout layout
	 * @param JPanel panelBase
	 * @param String success_panel_name
	 * @param String error_panel_name
	 */
	CustomDialog(JFrame frame) {
		// ==========================================================================
		// 初期化
		// ==========================================================================
		super(frame);
		this.mainFrame = frame;

		// ==========================================================================
		// 各ダイアログ共通の処理
		// ==========================================================================
		// モーダル化
		setModal(true);

		// 背景色
		getContentPane().setBackground(Color.BLACK);

		// レイアウト無し
		setLayout(null);

		// フレームのサイズを固定
		setResizable(false);

		// タイトルバー無し
		setUndecorated(true);
	}

	/**
	 * スマートゲートウェイの検索画面：スマートゲートウェイを検索していますダイアログ
	 * @throws InterruptedException
	 */
	public void searchExecutionDialog() {
		// タイトル
		setTitle("スマートゲートウェイを検索しています");

		// ダイアログのサイズ
		setSize(440, 115);

		// 画面中央に表示
		setLocationRelativeTo(null);

		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		JPanel panel = new JPanel();
		panel.setLayout(null);
		panel.setBounds(5, 5, 430, 105);
		panel.setBackground(Color.BLACK);
		Border outside = BorderFactory.createMatteBorder(2, 2, 2, 2, Color.WHITE);
		Border inside = BorderFactory.createEmptyBorder(0, 0, 0, 0);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));
		getContentPane().add(panel);

		// ローディング画像
		ImageIcon loading = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_ICON_LOADING));
		JLabel label_loading = new JLabel(loading);
		label_loading.setBounds(8, 25, 420, 20);
		label_loading.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_loading);

		// メッセージ
		JLabel label_msg = new JLabel("スマートゲートウェイを検索しています");
		label_msg.setBounds(8, 65, 430, 20);
		label_msg.setForeground(Color.WHITE);
		label_msg.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg);

		setVisible(true);

		System.out.println("ダイアログ表示");

		// スマートゲートウェイの検索処理
		SgwSearchThread thread = new SgwSearchThread();
		thread.setFrame(this.mainFrame);
		
		thread.start();

		System.out.println("スレッド開始");

		//	thread.join();
		System.out.println("★★★★★ スレッド終了 ★★★★★");
		setVisible(false);

		JButton btn_ok = new JButton("OK");
		btn_ok.addActionListener(this);
		btn_ok.setActionCommand("OK");
		btn_ok.setBounds(150, 80, 60, 20);
		panel.add(btn_ok);

		JButton btn_ng = new JButton("NG");
		btn_ng.addActionListener(this);
		btn_ng.setActionCommand("NG");
		btn_ng.setBounds(220, 80, 60, 20);
		panel.add(btn_ng);
	}

	/**
	 * スマートゲートウェイの検索画面：検索結果ダイアログ(成功)
	 */
	public void searchSuccessDialog() {
		// タイトル
		setTitle("検索結果");

		// ダイアログのサイズ
		setSize(440, 200);

		// 画面中央に表示
		setLocationRelativeTo(null);

		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		JPanel panel = new JPanel();
		panel.setLayout(null);
		panel.setBounds(5, 5, 430, 190);
		panel.setBackground(Color.BLACK);
		Border outside = BorderFactory.createMatteBorder(2, 2, 2, 2, Color.WHITE);
		Border inside = BorderFactory.createEmptyBorder(0, 0, 0, 0);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));
		getContentPane().add(panel);

		// 検索結果アイコン画像
		ImageIcon icon = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_ICON_RESULT1));
		JLabel label_icon = new JLabel(icon);
		label_icon.setBounds(10, 10, 32, 32);
		panel.add(label_icon);

		// タイトル
		JLabel label_title = new JLabel("検索結果");
		label_title.setBounds(50, 10, 200, 32);
		label_title.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
		label_title.setForeground(Color.WHITE);
		panel.add(label_title);

		// セパレータ
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 420, 15);
		panel.add(separator);

		// メッセージ
		JLabel label_msg = new JLabel("スマートゲートウェイが見つかりました");
		label_msg.setBounds(8, 65, 430, 20);
		label_msg.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 14));
		label_msg.setForeground(Color.WHITE);
		label_msg.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg);

		// 確認ボタン
		ImageIcon icon_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_OFF));
		ImageIcon rollover_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_ON));
		JButton btn_confirm = new JButton(icon_confirm);
		btn_confirm.setRolloverIcon(rollover_confirm);
		btn_confirm.setPressedIcon(rollover_confirm);
		btn_confirm.setBounds(170, 110, 94, 49);
		btn_confirm.setBorderPainted(false);
		btn_confirm.setContentAreaFilled(false);
		btn_confirm.addActionListener(this);
		btn_confirm.setActionCommand("search_success");
		panel.add(btn_confirm);
	}

	/**
	 * スマートゲートウェイの検索画面：検索結果失敗ダイアログ(失敗)
	 */
	public void searchErrorDialog() {
		// タイトル
		setTitle("検索結果");

		// ダイアログのサイズ
		setSize(440, 350);

		// 画面中央に表示
		setLocationRelativeTo(null);

		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		JPanel panel = new JPanel();
		panel.setLayout(null);
		panel.setBounds(5, 5, 430, 340);
		panel.setBackground(Color.BLACK);
		Border outside = BorderFactory.createMatteBorder(2, 2, 2, 2, Color.WHITE);
		Border inside = BorderFactory.createEmptyBorder(0, 0, 0, 0);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));
		getContentPane().add(panel);

		// 検索結果アイコン画像
		ImageIcon icon = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_ICON_RESULT2));
		JLabel label_icon = new JLabel(icon);
		label_icon.setBounds(10, 10, 32, 32);
		panel.add(label_icon);

		// タイトル
		JLabel label_title = new JLabel("検索結果");
		label_title.setBounds(50, 10, 200, 32);
		label_title.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
		label_title.setForeground(Color.WHITE);
		panel.add(label_title);

		// セパレータ
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 420, 15);
		panel.add(separator);

		// メッセージ1
		JLabel label_msg1 = new JLabel("スマートゲートウェイが見つかりません");
		label_msg1.setBounds(8, 65, 420, 20);
		label_msg1.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 14));
		label_msg1.setForeground(Color.WHITE);
		label_msg1.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg1);

		// メッセージ2
		JLabel label_msg2 = new JLabel("《  ご確認ください  》");
		label_msg2.setBounds(8, 105, 420, 20);
		label_msg2.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		label_msg2.setForeground(Color.WHITE);
		label_msg2.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg2);

		// メッセージ3
		String msg = "<html><body><ul>" +
				"<li>スマートゲートウェイのLANが点灯/点滅していますか？<br />" +
				"消灯している場合は、LANケーブルが接続されていないか、<br />" +
				"LANケーブルの接続先相手の機器の電源がOFF状態である<br />" +
				"可能性があります。</li>" +
				"<li>操作端末とスマートゲートウェイが同じネットワークに<br />" +
				"接続されていますか？</li>" +
				"</ul></body></html>";
		JLabel label_msg3 = new JLabel(msg);
		label_msg3.setBounds(0, 140, 400, 120);
		label_msg3.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		label_msg3.setForeground(Color.WHITE);
		panel.add(label_msg3);

		// 確認ボタン
		ImageIcon icon_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_OFF));
		ImageIcon rollover_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_ON));
		JButton btn_confirm = new JButton(icon_confirm);
		btn_confirm.setRolloverIcon(rollover_confirm);
		btn_confirm.setPressedIcon(rollover_confirm);
		btn_confirm.setBounds(170, 280, 94, 49);
		btn_confirm.setBorderPainted(false);
		btn_confirm.setContentAreaFilled(false);
		btn_confirm.addActionListener(this);
		btn_confirm.setActionCommand("search_error");
		panel.add(btn_confirm);
	}

	/**
	 * スマートゲートウェイの検索画面：検索結果失敗2ダイアログ(失敗)
	 */
	public void errorDialog() {
		// タイトル
		setTitle("エラー");

		// ダイアログのサイズ
		setSize(440, 300);

		// 画面中央に表示
		setLocationRelativeTo(null);

		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		JPanel panel = new JPanel();
		panel.setLayout(null);
		panel.setBounds(5, 5, 430, 290);
		panel.setBackground(Color.BLACK);
		Border outside = BorderFactory.createMatteBorder(2, 2, 2, 2, Color.WHITE);
		Border inside = BorderFactory.createEmptyBorder(0, 0, 0, 0);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));
		getContentPane().add(panel);

		// 検索結果アイコン画像
		ImageIcon icon = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_ICON_RESULT2));
		JLabel label_icon = new JLabel(icon);
		label_icon.setBounds(10, 10, 32, 32);
		panel.add(label_icon);

		// タイトル
		JLabel label_title = new JLabel("エラー");
		label_title.setBounds(50, 10, 200, 32);
		label_title.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
		label_title.setForeground(Color.WHITE);
		panel.add(label_title);

		// セパレータ
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 420, 15);
		panel.add(separator);

		// メッセージ1
		JLabel label_msg1 = new JLabel("WiFi《 無線LAN 》への接続が確認できませんでした");
		label_msg1.setBounds(8, 65, 420, 20);
		label_msg1.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 14));
		label_msg1.setForeground(Color.WHITE);
		label_msg1.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg1);

		// メッセージ2
		JLabel label_msg2 = new JLabel("《  ご確認ください  》");
		label_msg2.setBounds(8, 100, 420, 20);
		label_msg2.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		label_msg2.setForeground(Color.WHITE);
		label_msg2.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg2);

		// メッセージ3
		String msg = "<html><body><ul>" +
				"<li>操作端末がネットワークに接続している事が<br />" +
				"確認できませんでした。端末をネットワークに接続してから、<br />" +
				"再度操作をおこなってください。</li>" +
				"</ul></body></html>";
		JLabel label_msg3 = new JLabel(msg);
		label_msg3.setBounds(0, 120, 400, 100);
		label_msg3.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		label_msg3.setForeground(Color.WHITE);
		panel.add(label_msg3);

		// 確認ボタン
		ImageIcon icon_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_OFF));
		ImageIcon rollover_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_ON));
		JButton btn_confirm = new JButton(icon_confirm);
		btn_confirm.setRolloverIcon(rollover_confirm);
		btn_confirm.setPressedIcon(rollover_confirm);
		btn_confirm.setBounds(170, 230, 94, 49);
		btn_confirm.setBorderPainted(false);
		btn_confirm.setContentAreaFilled(false);
		btn_confirm.addActionListener(this);
		btn_confirm.setActionCommand("search_error");
		panel.add(btn_confirm);
	}

	/**
	 * 会員登録画面：会員登録失敗ダイアログ
	 */
	public void errorKaiinDialog() {
		// タイトル
		//setTitle("会員登録失敗エラー");
		setTitle("利用登録失敗エラー");

		// ダイアログのサイズ
		setSize(440, 300);

		// 画面中央に表示
		setLocationRelativeTo(null);

		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		JPanel panel = new JPanel();
		panel.setLayout(null);
		panel.setBounds(5, 5, 430, 290);
		panel.setBackground(Color.BLACK);
		Border outside = BorderFactory.createMatteBorder(2, 2, 2, 2, Color.WHITE);
		Border inside = BorderFactory.createEmptyBorder(0, 0, 0, 0);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));
		getContentPane().add(panel);

		// 検索結果アイコン画像
		ImageIcon icon = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_ICON_RESULT2));
		JLabel label_icon = new JLabel(icon);
		label_icon.setBounds(10, 10, 32, 32);
		panel.add(label_icon);

		// タイトル
		JLabel label_title = new JLabel("エラー");
		label_title.setBounds(50, 10, 200, 32);
		label_title.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
		label_title.setForeground(Color.WHITE);
		panel.add(label_title);

		// セパレータ
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 420, 15);
		panel.add(separator);

		// メッセージ1
		JLabel label_msg1 = new JLabel("会員が登録できませんでした");
		label_msg1.setBounds(8, 65, 420, 20);
		label_msg1.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 14));
		label_msg1.setForeground(Color.WHITE);
		label_msg1.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg1);

		// メッセージ2
		JLabel label_msg2 = new JLabel("《  ご確認ください  》");
		label_msg2.setBounds(8, 100, 420, 20);
		label_msg2.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		label_msg2.setForeground(Color.WHITE);
		label_msg2.setHorizontalAlignment(JLabel.CENTER);
		panel.add(label_msg2);

		// メッセージ3
		String msg = "<html><body><ul>" +
				"<li>会員登録部分は未作成です</li>" +
				"</ul></body></html>";
		JLabel label_msg3 = new JLabel(msg);
		label_msg3.setBounds(0, 120, 400, 100);
		label_msg3.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		label_msg3.setForeground(Color.WHITE);
		panel.add(label_msg3);

		// 確認ボタン
		ImageIcon icon_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_OFF));
		ImageIcon rollover_confirm = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_BUTTON_CONFIRM_ON));
		JButton btn_confirm = new JButton(icon_confirm);
		btn_confirm.setRolloverIcon(rollover_confirm);
		btn_confirm.setPressedIcon(rollover_confirm);
		btn_confirm.setBounds(170, 230, 94, 49);
		btn_confirm.setBorderPainted(false);
		btn_confirm.setContentAreaFilled(false);
		btn_confirm.addActionListener(this);
		btn_confirm.setActionCommand("regist_error");
		panel.add(btn_confirm);
	}

	/**
	 * アクションのイベント設定
	 *
	 * @param ActionEvent e イベント情報
	 */
	public void actionPerformed(ActionEvent e) {
		// コマンド名を取得
		String cmd = e.getActionCommand();

		// ========================================================================================================
		// スマートゲートウェイの検索
		// ========================================================================================================
		if (cmd.equals("OK")) {
			// ダイアログを閉じる
			setVisible(false);

			// エラー表示のダイアログを開く
			//			CustomDialog dlg = new CustomDialog(this.frame);
			//			dlg.searchSuccessDialog();

		} else if (cmd.equals("NG")) {
			// ダイアログを閉じる
			setVisible(false);

		} else if (cmd.equals("search_success")) {
			setVisible(false);

		} else if (cmd.equals("search_error")) {
			setVisible(false);

			// ========================================================================================================
			// 会員登録
			// ========================================================================================================
		} else if (cmd.equals("regist_error")) {
			setVisible(false);
		}
	}

	public void windowOpened(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void windowClosing(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void windowClosed(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void windowIconified(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void windowDeiconified(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void windowActivated(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void windowDeactivated(WindowEvent e) {
		// TODO 自動生成されたメソッド・スタブ

	}
}
