package com.SgwMiniSetup.sample;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

/**
 * スマートゲートウェイ 簡単セットアップGUI(PC版) - 電気料金換算設定ダイアログ
 *
 * @author Copyright (c) 2016 Enegate Co, ltd.
 * @version 1.xx
 */
public class ElecRateConversionSetting extends JDialog implements ActionListener {

	/**
	 * アプリの定数
	 */
	private static final long serialVersionUID = 1L;

	private final static String DIALOG_ELEC_RATE_CONVERSION_TITLE = "電気料金換算設定"; // タイトル
	private final static int DIALOG_ELEC_RATE_CONVERSION_WIDTH = 600; // 幅
	private final static int DIALOG_ELEC_RATE_CONVERSION_HEIGHT = 350; // 高さ

	private final static Color DIALOG_ELEC_RATE_CONVERSION_BG_COLOR = new Color(0, 0, 0); // 背景色(黒系:#000000)
	private final static Color DIALOG_ELEC_RATE_CONVERSION_BORDER_COLOR = new Color(255, 255, 255); // 枠線色(白系:#FFFFFF)
	private final static Color DIALOG_ELEC_RATE_CONVERSION_FONT_COLOR = new Color(255, 255, 255); // 文字色(白系:#FFFFFF)

	// 枠線の太さ
	private final static int DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_INSIDE = 0; // 内側
	private final static int DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_OUTSIDE = 2; // 外側

	// 文字の種別
	private final static String DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_TITLE = Font.SANS_SERIF; // タイトル
	private final static String DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG1 = Font.SANS_SERIF; // 本文1
	private final static String DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG2 = Font.SANS_SERIF; // 本文2
	private final static String DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_TXT = Font.SANS_SERIF; // テキスト

	// 文字の太さ
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_TITLE = Font.BOLD; // タイトル
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_MSG1 = Font.BOLD; // 本文1
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_MSG2 = Font.BOLD; // 本文2
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_TXT = Font.PLAIN; // テキスト

	// 文字のサイズ
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_TITLE = 20; // タイトル
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_MSG1 = 16; // 本文1
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_MSG2 = 16; // 本文2
	private final static int DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_TXT = 16; // テキスト

	// ========================================================================================================
	// コンポーネント配置(単位：ピクセル)
	// ========================================================================================================
	// パネル
	private final static int POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_X = 5; // X座標
	private final static int POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_Y = 5; // Y座標
	private final static int POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_WIDTH = 590; // 幅
	private final static int POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_HEIGHT = 340; // 高さ

	// メッセージ1
	private final static String DIALOG_ELEC_RATE_CONVERSION_MSG1 = "買っている電力の料金を変更します。";
	// メッセージ2
	private final static String DIALOG_ELEC_RATE_CONVERSION_MSG2 = "2桁（1～99）で入力の上、変更ボタンを押してください。";

	// ラベル1
	private final static String DIALOG_ELEC_RATE_CONVERSION_LBL1 = "1 kWhあたり";

	// ラベル2
	private final static String DIALOG_ELEC_RATE_CONVERSION_LBL2 = "円   （ 初期値: 22 ）";

	private final static int NO_VALUE = -1;
	private int curElecRateValue = NO_VALUE;

	/**
	 * コンストラクタ
	 *
	 * @param JFrame
	 *            frame
	 * @param CardLayout
	 *            layout
	 * @param JPanel
	 *            panelBase
	 * @param String
	 *            success_panel_name
	 * @param String
	 *            error_panel_name
	 */

	JTextField text1;

	public ElecRateConversionSetting(JFrame frame, String value) {

		// ========================================================================================================
		// 初期化
		// ========================================================================================================
		// 親フレーム継承
		super(frame);
		setElecRateConversionSetting(value);

		// ========================================================================================================
		// 各ダイアログ共通の処理
		// ========================================================================================================
		setTitle(DIALOG_ELEC_RATE_CONVERSION_TITLE);
		setSize(DIALOG_ELEC_RATE_CONVERSION_WIDTH, DIALOG_ELEC_RATE_CONVERSION_HEIGHT); // 幅、高さ
		getContentPane().setBackground(DIALOG_ELEC_RATE_CONVERSION_BG_COLOR); // 背景色
		setLayout(null); // レイアウト無し
		setResizable(false); // フレームのサイズを固定
		setUndecorated(true); // タイトルバー無し
		setLocationRelativeTo(frame); // 親フレームの画面中央に表示
		setModal(true); // モーダル化

		// ========================================================================================================
		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		// ========================================================================================================
		JPanel panel = new JPanel();
		panel.setLayout(null);

		// X座標、Y座標、幅、高さをセット
		panel.setBounds(POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_X, POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_Y,
				POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_WIDTH, POS_DIALOG_ELEC_RATE_CONVERSION_PANEL_HEIGHT);

		// 背景色
		panel.setBackground(DIALOG_ELEC_RATE_CONVERSION_BG_COLOR);

		// 外側の枠線の太さ、内側の枠線の太さをセット
		Border outside = BorderFactory.createMatteBorder(
				DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_OUTSIDE, DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_OUTSIDE,
				DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_OUTSIDE, DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_OUTSIDE,
				DIALOG_ELEC_RATE_CONVERSION_BORDER_COLOR);
		Border inside = BorderFactory.createEmptyBorder(
				DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_INSIDE, DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_INSIDE,
				DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_INSIDE, DIALOG_ELEC_RATE_CONVERSION_BORDER_WEIGHT_INSIDE);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));

		getContentPane().add(panel);

		// ========================================================================================================
		// コンポーネント作成
		// ========================================================================================================

		// タイトル
		JLabel label_title = new JLabel(DIALOG_ELEC_RATE_CONVERSION_TITLE);
		label_title.setBounds(30, 10, 200, 32);
		label_title.setFont(new Font(DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_TITLE,
				DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_TITLE, DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_TITLE));
		label_title.setForeground(DIALOG_ELEC_RATE_CONVERSION_FONT_COLOR);
		panel.add(label_title);

		// 区切り線
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 580, 15);
		panel.add(separator);

		// メッセージ1
		JLabel label_explanation1 = new JLabel(DIALOG_ELEC_RATE_CONVERSION_MSG1);
		label_explanation1.setBounds(30, 75, 530, 20);
		label_explanation1.setFont(new Font(
				DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG1, DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_MSG1,
				DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_MSG1));
		label_explanation1.setForeground(DIALOG_ELEC_RATE_CONVERSION_FONT_COLOR);
		panel.add(label_explanation1);

		// メッセージ2
		JLabel label_explanation2 = new JLabel(DIALOG_ELEC_RATE_CONVERSION_MSG2);
		label_explanation2.setBounds(30, 105, 530, 20);
		label_explanation2.setFont(new Font(
				DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG2, DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_MSG2,
				DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_MSG2));
		label_explanation2.setForeground(DIALOG_ELEC_RATE_CONVERSION_FONT_COLOR);
		panel.add(label_explanation2);

		//テキストフィールド
		text1 = new JTextField(Integer.toString(curElecRateValue), 2);
		text1.setBounds(230, 175, 60, 30);
		text1.setFont(new Font(DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_TXT, DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_TXT, DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_TXT));
		panel.add(text1);

		//ラベル
		JLabel label1 = new JLabel(DIALOG_ELEC_RATE_CONVERSION_LBL1);
		label1.setBounds(125, 175, 200, 30);
		label1.setFont(new Font(
				DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG2, DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_MSG2,
				DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_MSG2));
		label1.setForeground(DIALOG_ELEC_RATE_CONVERSION_FONT_COLOR);
		panel.add(label1);

		//ラベル
		JLabel label2 = new JLabel(DIALOG_ELEC_RATE_CONVERSION_LBL2);
		label2.setBounds(300, 175, 260, 30);
		label2.setFont(new Font(
				DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG2, DIALOG_ELEC_RATE_CONVERSION_FONT_WEIGHT_MSG2,
				DIALOG_ELEC_RATE_CONVERSION_FONT_SIZE_MSG2));
		label2.setForeground(DIALOG_ELEC_RATE_CONVERSION_FONT_COLOR);
		panel.add(label2);

		// OKボタン
		JButton btn_ok = new JButton("変更");
		btn_ok.setBounds(175, 280, 120, 40);
		btn_ok.setFont(new Font(DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG1, 1, 16));
		btn_ok.addActionListener(this);
		btn_ok.setActionCommand("Yes");
		panel.add(btn_ok);

		// キャンセルボタン
		JButton btn_cancel = new JButton("キャンセル");
		btn_cancel.setBounds(305, 280, 120, 40);
		btn_cancel.setFont(new Font(DIALOG_ELEC_RATE_CONVERSION_FONT_STYLE_MSG1, 1, 16));
		btn_cancel.addActionListener(this);
		btn_cancel.setActionCommand("No");
		panel.add(btn_cancel);
	}

	/**
	 * アクションのイベント設定
	 *
	 * @param ActionEvent
	 *            e イベント情報
	 */
	public void actionPerformed(ActionEvent e) {
		// コマンド名を取得
		String cmd = e.getActionCommand();

		// 「はい」ボタンを押した場合
		if (cmd.equals("Yes")) {
			if (checkValue(text1.getText())) {
				try {
					curElecRateValue = Integer.parseInt(text1.getText());
				} catch (Exception ex) {
					curElecRateValue = NO_VALUE;
				}
			}else {
				curElecRateValue = NO_VALUE;
			}

			if (curElecRateValue != NO_VALUE) {
				// 非表示にする
				setVisible(false);
			}
		}
		// 「いいえ」ボタンを押した場合
		else if (cmd.equals("No")) {
			curElecRateValue = NO_VALUE;

			// 非表示にする
			setVisible(false);
		}
	}

	public void setElecRateConversionSetting(String value) {
		try {
			curElecRateValue = Integer.parseInt(value);
		} catch (Exception e) {
			curElecRateValue = NO_VALUE;
		}
	}

	public int getElecRateConversionSetting() {
		return curElecRateValue;
	}

	private boolean checkValue(String value) {
		try {
			//2文字以内
			int i = value.length();
			if (i > 2) {
				JOptionPane.showMessageDialog(this, "桁数が不正です。", "確認", JOptionPane.INFORMATION_MESSAGE);
				return false;
			}

			// ゼロは不整値扱い
			int num = Integer.parseInt(value);

			//半角数値
			Pattern p = Pattern.compile("^[0-9]*$");
			Matcher m = p.matcher(value);
			if (m.find() && (0 < num)) {
			} else {
				JOptionPane.showMessageDialog(this, "電気料金換算値が不正です。", "確認", JOptionPane.INFORMATION_MESSAGE);
				return false;
			}
		} catch (NumberFormatException numberFormatException) {
			return false;
		}
		return true;
	}
}
