package com.SgwMiniSetup.sample;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

/**
 * スマートゲートウェイ 簡単セットアップGUI(PC版) - 宅外家電制御設定ダイアログ
 *
 * @author Copyright (c) 2013 Enegate Co, ltd.
 * @version 1.00
 */
public class RemoteControlSettingDialog extends JDialog implements ActionListener {
	/**
	 * アプリの定数
	 */
	private static final long serialVersionUID = 1L;

	// ========================================================================================================
	// 各パネルの設定値
	// ========================================================================================================
	private JFrame frame; // 親フレーム情報

	private final static String DIALOG_REMOTE_CONTROL_TITLE = "宅外家電操作設定"; // タイトル
	private final static int DIALOG_REMOTE_CONTROL_WIDTH = 600; // 幅
	private final static int DIALOG_REMOTE_CONTROL_HEIGHT = 350; // 高さ

	private final static Color DIALOG_REMOTE_CONTROL_BG_COLOR = new Color(0, 0, 0); // 背景色(黒系:#000000)
	private final static Color DIALOG_REMOTE_CONTROL_BORDER_COLOR = new Color(255, 255, 255); // 枠線色(白系:#FFFFFF)
	private final static Color DIALOG_REMOTE_CONTROL_FONT_COLOR = new Color(255, 255, 255); // 文字色(白系:#FFFFFF)

	// 枠線の太さ
	private final static int DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_INSIDE = 0; // 内側
	private final static int DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_OUTSIDE = 2; // 外側

	// 文字の種別
	private final static String DIALOG_REMOTE_CONTROL_FONT_STYLE_TITLE = Font.SANS_SERIF; // タイトル
	private final static String DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG1 = Font.SANS_SERIF; // 本文1
	private final static String DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG2 = Font.SANS_SERIF; // 本文2

	// 文字の太さ
	private final static int DIALOG_REMOTE_CONTROL_FONT_WEIGHT_TITLE = Font.BOLD; // タイトル
	private final static int DIALOG_REMOTE_CONTROL_FONT_WEIGHT_MSG1 = Font.BOLD; // 本文1
	private final static int DIALOG_REMOTE_CONTROL_FONT_WEIGHT_MSG2 = Font.BOLD; // 本文2

	// 文字のサイズ
	private final static int DIALOG_REMOTE_CONTROL_FONT_SIZE_TITLE = 20; // タイトル
	private final static int DIALOG_REMOTE_CONTROL_FONT_SIZE_MSG1 = 14; // 本文1
	private final static int DIALOG_REMOTE_CONTROL_FONT_SIZE_MSG2 = 12; // 本文2

	// ========================================================================================================
	// コンポーネント配置(単位：ピクセル)
	// ========================================================================================================
	// パネル
	private final static int POS_DIALOG_REMOTE_CONTROL_PANEL_X = 5; // X座標
	private final static int POS_DIALOG_REMOTE_CONTROL_PANEL_Y = 5; // Y座標
	private final static int POS_DIALOG_REMOTE_CONTROL_PANEL_WIDTH = 590; // 幅
	private final static int POS_DIALOG_REMOTE_CONTROL_PANEL_HEIGHT = 340; // 高さ

	// メッセージ1
	private final static String DIALOG_REMOTE_CONTROL_MSG1 = "宅外からの家電操作設定を変更しますか？"; // 文言
	// メッセージ2
	private final static String DIALOG_REMOTE_CONTROL_MSG2 = "<html>宅外家電操作設定を許可すると、サーバを介してのECHONET Lite対応家電機器への<br>" +
															"操作が可能になりますが、インターネット回線の異常が24時間継続した場合、自動的に<br>" +
															"エアコンOFF操作を行います。"; // 文言

	private final static String CONTROL_DISABLED = "許可しない";
	private final static String CONTROL_ENABLED = "許可する";
	private final static String NO_SELECTED = "(未設定)";

	private final static int TYPE_CONTROL_DISABLED = 0;
	private final static int TYPE_CONTROL_ENABLED = 1;
	private final static int TYPE_NO_SELECTED = -1;

	private int curRemoteControlSetting = TYPE_NO_SELECTED;


	/**
	 * コンストラクタ
	 *
	 * @param JFrame
	 *            frame
	 * @param CardLayout
	 *            layout
	 * @param JPanel
	 *            panelBase
	 * @param String
	 *            success_panel_name
	 * @param String
	 *            error_panel_name
	 */
	public RemoteControlSettingDialog(JFrame frame, String item) {

		// ========================================================================================================
		// 初期化
		// ========================================================================================================
		// 親フレーム継承
		super(frame);
		this.frame = frame;

		setRemoteControlSetting(item);

		// ========================================================================================================
		// 各ダイアログ共通の処理
		// ========================================================================================================
		setTitle(DIALOG_REMOTE_CONTROL_TITLE);
		setSize(DIALOG_REMOTE_CONTROL_WIDTH, DIALOG_REMOTE_CONTROL_HEIGHT); // 幅、高さ
		getContentPane().setBackground(DIALOG_REMOTE_CONTROL_BG_COLOR); // 背景色
		setLayout(null); // レイアウト無し
		setResizable(false); // フレームのサイズを固定
		setUndecorated(true); // タイトルバー無し
		setLocationRelativeTo(frame); // 親フレームの画面中央に表示
		setModal(true); // モーダル化

		// ========================================================================================================
		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		// ========================================================================================================
		JPanel panel = new JPanel();
		panel.setLayout(null);

		// X座標、Y座標、幅、高さをセット
		panel.setBounds(POS_DIALOG_REMOTE_CONTROL_PANEL_X, POS_DIALOG_REMOTE_CONTROL_PANEL_Y,
				POS_DIALOG_REMOTE_CONTROL_PANEL_WIDTH, POS_DIALOG_REMOTE_CONTROL_PANEL_HEIGHT);

		// 背景色
		panel.setBackground(DIALOG_REMOTE_CONTROL_BG_COLOR);

		// 外側の枠線の太さ、内側の枠線の太さをセット
		Border outside = BorderFactory.createMatteBorder(
				DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_OUTSIDE, DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_OUTSIDE,
				DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_OUTSIDE, DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_OUTSIDE,
				DIALOG_REMOTE_CONTROL_BORDER_COLOR);
		Border inside = BorderFactory.createEmptyBorder(
				DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_INSIDE, DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_INSIDE,
				DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_INSIDE, DIALOG_REMOTE_CONTROL_BORDER_WEIGHT_INSIDE);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));

		getContentPane().add(panel);

		// ========================================================================================================
		// コンポーネント作成
		// ========================================================================================================

		// タイトル
		JLabel label_title = new JLabel(DIALOG_REMOTE_CONTROL_TITLE);
		label_title.setBounds(30, 10, 200, 32);
		label_title.setFont(new Font(DIALOG_REMOTE_CONTROL_FONT_STYLE_TITLE,
				DIALOG_REMOTE_CONTROL_FONT_WEIGHT_TITLE, DIALOG_REMOTE_CONTROL_FONT_SIZE_TITLE));
		label_title.setForeground(DIALOG_REMOTE_CONTROL_FONT_COLOR);
		panel.add(label_title);

		// 区切り線
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 580, 15);
		panel.add(separator);

		String selectedItem = NO_SELECTED;
		boolean btnCtrlDisabledEnabled = false;
		boolean btnCtrlEnabledEnabled = false;
		if (curRemoteControlSetting == TYPE_CONTROL_DISABLED) {
			selectedItem = CONTROL_DISABLED;
			btnCtrlDisabledEnabled = true;
		} else if (curRemoteControlSetting == TYPE_CONTROL_ENABLED) {
			selectedItem = CONTROL_ENABLED;
			btnCtrlEnabledEnabled = true;
		}

		// メッセージ1
		JLabel label_explanation1 = new JLabel(DIALOG_REMOTE_CONTROL_MSG1 + "        ※現在の設定: [ " + selectedItem + " ]");
		label_explanation1.setBounds(30, 65, 530, 20);
		label_explanation1.setFont(new Font(
				DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG1, DIALOG_REMOTE_CONTROL_FONT_WEIGHT_MSG1,
				DIALOG_REMOTE_CONTROL_FONT_SIZE_MSG1));
		label_explanation1.setForeground(DIALOG_REMOTE_CONTROL_FONT_COLOR);
		panel.add(label_explanation1);

		ButtonGroup group = new ButtonGroup();

		// オプション１
		JRadioButton radioBtnDisanbled = new JRadioButton(CONTROL_DISABLED, btnCtrlDisabledEnabled);
		radioBtnDisanbled.setBounds(50, 110, 530, 20);
		radioBtnDisanbled.setFont(new Font(DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG1, 1, 18));
		radioBtnDisanbled.setForeground(DIALOG_REMOTE_CONTROL_FONT_COLOR);
		radioBtnDisanbled.setBackground(DIALOG_REMOTE_CONTROL_BG_COLOR);
		radioBtnDisanbled.addActionListener(this);
		radioBtnDisanbled.setActionCommand("ctrl_disable");
		group.add(radioBtnDisanbled);
		panel.add(radioBtnDisanbled);
		// オプション２
		JRadioButton radioBtnEnabled = new JRadioButton(CONTROL_ENABLED, btnCtrlEnabledEnabled);
		radioBtnEnabled.setBounds(50, 150, 530, 20);
		radioBtnEnabled.setFont(new Font(DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG1, 1, 18));
		radioBtnEnabled.setForeground(DIALOG_REMOTE_CONTROL_FONT_COLOR);
		radioBtnEnabled.setBackground(DIALOG_REMOTE_CONTROL_BG_COLOR);
		radioBtnEnabled.addActionListener(this);
		radioBtnEnabled.setActionCommand("ctrl_enable");
		group.add(radioBtnEnabled);
		panel.add(radioBtnEnabled);

		// メッセージ2
		JLabel label_explanation2 = new JLabel(DIALOG_REMOTE_CONTROL_MSG2);
		label_explanation2.setBounds(30, 185, 530, 80);
		label_explanation2.setFont(new Font(
				DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG2, DIALOG_REMOTE_CONTROL_FONT_WEIGHT_MSG2,
				DIALOG_REMOTE_CONTROL_FONT_SIZE_MSG2));
		label_explanation2.setForeground(DIALOG_REMOTE_CONTROL_FONT_COLOR);
		panel.add(label_explanation2);

		// OKボタン
		JButton btn_ok = new JButton("はい");
		btn_ok.setBounds(175, 280, 120, 40);
		btn_ok.setFont(new Font(DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG1, 1, 16));
		btn_ok.addActionListener(this);
		btn_ok.setActionCommand("Yes");
		panel.add(btn_ok);

		// キャンセルボタン
		JButton btn_cancel = new JButton("いいえ");
		btn_cancel.setBounds(305, 280, 120, 40);
		btn_cancel.setFont(new Font(DIALOG_REMOTE_CONTROL_FONT_STYLE_MSG1, 1, 16));
		btn_cancel.addActionListener(this);
		btn_cancel.setActionCommand("No");
		panel.add(btn_cancel);
	}

	/**
	 * アクションのイベント設定
	 *
	 * @param ActionEvent
	 *            e イベント情報
	 */
	public void actionPerformed(ActionEvent e) {
		// コマンド名を取得
		String cmd = e.getActionCommand();

		// 「はい」ボタンを押した場合
		if (cmd.equals("Yes")) {
			// 非表示にする
			setVisible(false);
		}
		// 「いいえ」ボタンを押した場合
		else if (cmd.equals("No")) {
			curRemoteControlSetting = TYPE_NO_SELECTED;

			// 非表示にする
			setVisible(false);
		}
		// オプションが選択された場合
		else if (cmd.equals("ctrl_disable")) {
			if (((JRadioButton) e.getSource()).isSelected()) {
				curRemoteControlSetting = TYPE_CONTROL_DISABLED;
			}
		}
		else if (cmd.equals("ctrl_enable")) {
			if (((JRadioButton) e.getSource()).isSelected()) {
				curRemoteControlSetting = TYPE_CONTROL_ENABLED;
			}
		}
	}

	public void setRemoteControlSetting(String type) {
		try {
			curRemoteControlSetting = Integer.parseInt(type);
		} catch (Exception e) {
			curRemoteControlSetting = TYPE_NO_SELECTED;
		}
	}

	public int getRemoteControlSetting() {
		return curRemoteControlSetting;
	}
}
