package com.SgwMiniSetup.sample;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingWorker;
import javax.swing.border.Border;

import com.SgwMiniSetup.thread.SgwSearchThread;
import com.SgwSetting.common.CommonParam;


/**
 * スマートゲートウェイ 簡単セットアップGUI(PC版) - スマートゲートウェイの検索ダイアログ(検索中)
 *
 * @author Copyright (c) 2013 Enegate Co, ltd.
 * @version 1.00
 */
public class SearchExecutionDialog extends JDialog implements ActionListener {
	/**
	 * アプリの定数
	 */
	private static final long serialVersionUID = 1L;

	// ========================================================================================================
	// 画像リソース
	// ========================================================================================================
	private final static String IMG_ICON_LOADING     = "res/img/loading.gif";      // ローディング画像
	private final static String FILE_SEARCH_SGW_PATH = "res/file/search_sgw.txt";  // SGWのIP検索結果を格納したファイル


	// ========================================================================================================
	// 各パネルの設定値
	// ========================================================================================================
	private JFrame frame;            // 親フレーム情報
	private SwingWorker<String, String> worker;   // バックグラウンドでSGW検索を行うプロセス


	private final static String DIALOG_SEARCH_EXECUTION_TITLE  = "スマートゲートウェイを検索しています";  // タイトル
	private final static int DIALOG_SEARCH_EXECUTION_WIDTH  = 440;                     // 幅
	private final static int DIALOG_SEARCH_EXECUTION_HEIGHT = 115;                     // 高さ

	private final static Color DIALOG_SEARCH_EXECUTION_BG_COLOR     = new Color(0, 0, 0);       // 背景色(黒系:#000000)
	private final static Color DIALOG_SEARCH_EXECUTION_BORDER_COLOR = new Color(255, 255, 255); // 枠線色(白系:#FFFFFF)
	private final static Color DIALOG_SEARCH_EXECUTION_FONT_COLOR   = new Color(255, 255, 255); // 文字色(白系:#FFFFFF)

	// 枠線の太さ
	private final static int DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_INSIDE  = 0;  // 内側
	private final static int DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_OUTSIDE = 2;  // 外側


	// ========================================================================================================
	// コンポーネント配置(単位：ピクセル)
	// ========================================================================================================
	// パネル
	private final static int POS_DIALOG_SEARCH_EXECUTION_PANEL_X      = 5;               // X座標
	private final static int POS_DIALOG_SEARCH_EXECUTION_PANEL_Y      = 5;               // Y座標
	private final static int POS_DIALOG_SEARCH_EXECUTION_PANEL_WIDTH  = 430;             // 幅
	private final static int POS_DIALOG_SEARCH_EXECUTION_PANEL_HEIGHT = 105;             // 高さ

	// ローディング画像
	private final static int POS_DIALOG_SEARCH_EXECUTION_LOADING_X      = 8;             // X座標
	private final static int POS_DIALOG_SEARCH_EXECUTION_LOADING_Y      = 25;            // Y座標
	private final static int POS_DIALOG_SEARCH_EXECUTION_LOADING_WIDTH  = 420;           // 幅
	private final static int POS_DIALOG_SEARCH_EXECUTION_LOADING_HEIGHT = 20;            // 高さ
	private final static int POS_DIALOG_SEARCH_EXECUTION_LOADING_ALIGN  = JLabel.CENTER; // 位置

	// メッセージ
	private final static int POS_DIALOG_SEARCH_EXECUTION_MSG_X      = 8;             // X座標
	private final static int POS_DIALOG_SEARCH_EXECUTION_MSG_Y      = 65;            // Y座標
	private final static int POS_DIALOG_SEARCH_EXECUTION_MSG_WIDTH  = 430;           // 幅
	private final static int POS_DIALOG_SEARCH_EXECUTION_MSG_HEIGHT = 20;            // 高さ
	private final static int POS_DIALOG_SEARCH_EXECUTION_MSG_ALIGN  = JLabel.CENTER; // 位置
	private final static String DIALOG_SEARCH_EXECUTION_MSG  = "スマートゲートウェイを検索しています";  // 文言
	
	private SgwSearchThread myThread;
	private JFrame myFrame;

	/**
	 * コンストラクタ
	 *
	 * @param JFrame frame
	 * @param CardLayout layout
	 * @param JPanel panelBase
	 * @param String success_panel_name
	 * @param String error_panel_name
	 */
	public SearchExecutionDialog(JFrame frame) {
		// ========================================================================================================
		// 初期化
		// ========================================================================================================
		// 親フレーム継承
		super(frame);
		this.myFrame = frame;

		// ========================================================================================================
		// 各ダイアログ共通の処理
		// ========================================================================================================
		setTitle(DIALOG_SEARCH_EXECUTION_TITLE);
		setSize(DIALOG_SEARCH_EXECUTION_WIDTH, DIALOG_SEARCH_EXECUTION_HEIGHT); // 幅、高さ
		getContentPane().setBackground(DIALOG_SEARCH_EXECUTION_BG_COLOR);       // 背景色
		setLayout(null);                  // レイアウト無し
		setResizable(false);              // フレームのサイズを固定
		setUndecorated(true);             // タイトルバー無し
		setLocationRelativeTo(frame);     // 親フレームの画面中央に表示
		setModal(true);                   // モーダル化

		// ========================================================================================================
		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		// ========================================================================================================
		JPanel panel = new JPanel();
		panel.setLayout(null);

		// X座標、Y座標、幅、高さをセット
		panel.setBounds(POS_DIALOG_SEARCH_EXECUTION_PANEL_X, POS_DIALOG_SEARCH_EXECUTION_PANEL_Y,
				POS_DIALOG_SEARCH_EXECUTION_PANEL_WIDTH, POS_DIALOG_SEARCH_EXECUTION_PANEL_HEIGHT);

		// 背景色
		panel.setBackground(DIALOG_SEARCH_EXECUTION_BG_COLOR);

		// 外側の枠線の太さ、内側の枠線の太さをセット
		Border outside = BorderFactory.createMatteBorder(
				DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_OUTSIDE, DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_OUTSIDE,
				DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_OUTSIDE, DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_OUTSIDE,
				DIALOG_SEARCH_EXECUTION_BORDER_COLOR);
		Border inside  = BorderFactory.createEmptyBorder(
				DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_INSIDE, DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_INSIDE,
				DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_INSIDE, DIALOG_SEARCH_EXECUTION_BORDER_WEIGHT_INSIDE);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));

		getContentPane().add(panel);

		// ========================================================================================================
		// ローディング画像
		// ========================================================================================================
		// 画像読み込み
		ImageIcon loading    = new ImageIcon(this.getClass().getClassLoader().getResource(IMG_ICON_LOADING));
		JLabel label_loading = new JLabel(loading);

		// X座標、Y座標、幅、高さをセット
		label_loading.setBounds(POS_DIALOG_SEARCH_EXECUTION_LOADING_X, POS_DIALOG_SEARCH_EXECUTION_LOADING_Y,
				POS_DIALOG_SEARCH_EXECUTION_LOADING_WIDTH, POS_DIALOG_SEARCH_EXECUTION_LOADING_HEIGHT);

		// 位置をセット
		label_loading.setHorizontalAlignment(POS_DIALOG_SEARCH_EXECUTION_LOADING_ALIGN);

		panel.add(label_loading);

		// ========================================================================================================
		// メッセージ
		// ========================================================================================================
		// 文言をセット
		JLabel label_msg = new JLabel(DIALOG_SEARCH_EXECUTION_MSG);

		// X座標、Y座標、幅、高さをセット
		label_msg.setBounds(POS_DIALOG_SEARCH_EXECUTION_MSG_X, POS_DIALOG_SEARCH_EXECUTION_MSG_Y,
				POS_DIALOG_SEARCH_EXECUTION_MSG_WIDTH, POS_DIALOG_SEARCH_EXECUTION_MSG_HEIGHT);

		// 文字色、位置をセット
		label_msg.setForeground(DIALOG_SEARCH_EXECUTION_FONT_COLOR);
		label_msg.setHorizontalAlignment(POS_DIALOG_SEARCH_EXECUTION_MSG_ALIGN);

		panel.add(label_msg);
		
		// 中止ボタン
		JButton btn_cancel = new JButton("中止");
		btn_cancel.setBounds(360, 65, 60, 28);
		btn_cancel.setFont(new Font("検索を中止します", 1, 12));
		btn_cancel.addActionListener(this);
		btn_cancel.setActionCommand("Close");
		panel.add(btn_cancel);

		// ========================================================================================================
		// バックグラウンドで処理する内容
		// ========================================================================================================
		// 実行結果Integer, 処理経過データint[]のSwingWorker
		final SwingWorker<String, int[]> sw = new SwingWorker<String, int[]>() {
			/**
			 * バックグラウンド処理
			 * @return
			 */
			@Override
			protected String doInBackground() throws Exception {
				// スマートゲートウェイの検索スレッド処理開始
				SgwSearchThread thread = new SgwSearchThread();
				thread.setFrame(myFrame);
				thread.start();
				myThread = thread;

				// スレッド終了待ち
				thread.join();

				return "";
			}

			/**
			 * 処理終了
			 */
			@Override
			protected void done() {
				// ダイアログを閉じる
				setVisible(false);
			}
		};


		// ========================================================================================================
		// ウィンドウ処理
		// ========================================================================================================
		/**
		 * ダイアログのWindowリスナー
		 */
		this.addWindowListener(new WindowListener() {
			/**
			 * 開かれた
			 */
			public void windowOpened(WindowEvent e) {
				// ダイアログが開かれたと同時にバックグラウンドでSwingWorkderを実行
				sw.execute();
			}


			/**
			 * 閉じられている
			 */
			public void windowClosing(WindowEvent e) {
			}


			/**
			 * 閉じた
			 */
			public void windowClosed(WindowEvent e) {
			}


			/**
			 * アイコン化された
			 */
			public void windowIconified(WindowEvent e) {
			}


			/**
			 * 非アイコン化された
			 */
			public void windowDeiconified(WindowEvent e) {
			}


			/**
			 * アクティブになった
			 */
			public void windowActivated(WindowEvent e) {
			}


			/**
			 * 非アクティブになった
			 */
			public void windowDeactivated(WindowEvent e) {
			}

		});
	}

	@SuppressWarnings("deprecation")
	public void actionPerformed(ActionEvent e) {
		try {
			// スレッドを中止
			if (myThread != null && myThread.isAlive()) {
				myThread.stopThread();
				// 初期設定に戻す
				CommonParam.stopSearchResult = true;
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}
