package com.SgwMiniSetup.sample;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.*;

/**
 * 
 * SGW検索結果の表示テーブル
 *
 *
 */
public class SearchResultTable extends JTable {
    
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected String[] cols = {
            "No", "IPアドレス", " ", "製造番号", "時刻設定", "サーバ名登録 ", "サーバ名確認", "宅外操作", "Encode", "更新日時"
    };

    protected boolean[] columnSortValues = {
        true, true, true, true, true
    };
    
    //タイトル設定
    public static String TABLE_HEADER_0 = "No";
    public static String TABLE_HEADER_1 = "IPアドレス";
    public static String TABLE_HEADER_2 = " ";
    public static String TABLE_HEADER_3 = "製造番号";
    public static String TABLE_HEADER_4 = "時刻設定";
    public static String TABLE_HEADER_5 = "サーバ名登録";
    public static String TABLE_HEADER_6 = "サーバ名確認";
    public static String TABLE_HEADER_7 = "宅外操作";
    public static String TABLE_HEADER_8 = "Encode";
    public static String TABLE_HEADER_9 = "更新日時";
    
    // テーブルヘッダー
    private JTableHeader header;
    
    // デコレータ
    private final SortDecorator decorator;

    /**
     * 
     * This method initializes
     * コンストラクタ
     * 
     */
    public  SearchResultTable() {
        super();
        
        //モデルを取得
        SearchResultTableModel model = new SearchResultTableModel(this, cols, 0);
        setModel(model);//モデルをセット
        setRowHeight(20);//カラム数をセット

        //ヘッダを取得
        //JTableHeader header = (JTableHeader) this.getTableHeader();
        header = (JTableHeader) this.getTableHeader();
        header.setUpdateTableInRealTime(true);
        header.setReorderingAllowed(false);

        //カラムモデルを取得
        DefaultTableColumnModel cmodel = (DefaultTableColumnModel) this.getColumnModel();
        TableColumn column = cmodel.getColumn(1);
        column.setCellRenderer(new SearchResultTableCellRenderer());

        // デコレータクラス（SortDecorator）によるソートを行う
        //final SortDecorator decorator = new SortDecorator(this.getModel());
        decorator = new SortDecorator(this.getModel());
        this.setModel(decorator);
        header.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                TableColumnModel tcm = getColumnModel();
                int vc = tcm.getColumnIndexAtX(e.getX());
                //System.out.println("... JTableHeader mouseClicked[" + vc + "]");
                int mc = convertColumnIndexToModel(vc);

                if (columnSortValues[mc]) {
                    decorator.sort(mc);     // 順ソート
                    columnSortValues[mc] = false;
                } else {
                    decorator.resort(mc);   // 逆ソート
                    columnSortValues[mc] = true;
                }

                // ヘッダーを更新する
                updateColumnHeader(mc);
                // 更新する
                repaint();
                // 再描画
                getThisTable().repaint();
            }
        });

        // 最初の列の幅を30にする
        this.getColumnModel().getColumn(0).setMaxWidth(30);
        // 2列目の幅の初期値を200にする
        this.getColumnModel().getColumn(1).setMaxWidth(100);
        // 3列目の幅の初期値を30にする
        this.getColumnModel().getColumn(2).setMaxWidth(25);
        // 4列目の幅の初期値を100にする
        this.getColumnModel().getColumn(3).setMaxWidth(70);
        // 5列目の幅の初期値を100にする
        this.getColumnModel().getColumn(4).setMaxWidth(65);
        // 6列目の幅の初期値を100にする
        this.getColumnModel().getColumn(5).setMaxWidth(90);
        // 7列目の幅の初期値を100にする
        this.getColumnModel().getColumn(6).setMaxWidth(200);
        // 9列目の幅の初期値を60にする
        this.getColumnModel().getColumn(7).setMaxWidth(60);
        // ヘッダを表示する
        this.tableHeader.setVisible(true);
        
        // オブジェクト格納用の欄は見せない
        header.getColumnModel().getColumn(8).setMaxWidth(0);
        header.getColumnModel().getColumn(8).setMinWidth(0);
        this.getColumnModel().getColumn(8).setMaxWidth(0);
        this.getColumnModel().getColumn(8).setMinWidth(0);
        header.getColumnModel().getColumn(9).setMaxWidth(0);
        header.getColumnModel().getColumn(9).setMinWidth(0);
        this.getColumnModel().getColumn(9).setMaxWidth(0);
        this.getColumnModel().getColumn(9).setMinWidth(0);
    }
    
    /**
     * 
     * テーブルをクリア
     * 
     */
    public void clear() {
        //モデルを取得
        SortDecorator model = (SortDecorator) this.getModel();
        int rows = model.getRowCount();
        for (int i = 0; i < rows; i++) {
            // model.removeRow(0);
            model.getTableModel().removeRow(0);
        }
    }
    
    /**
     * 
     * すべてのカラムを有効（無効）にします。
     * 
     * @param isEnabled すべてのタスクの有効（無効）を指定します。
     */
    public void setEnabledTable(boolean isEnabled) {
        TableModel model = this.getModel();
        int rows = model.getRowCount();
        for (int i = 0; i < rows; i++) {
            boolean tmpEnabled = isEnabled;
            //if (!model.isCellEditable(i, 7)) {
            //    tmpEnabled = false;  // 編集不可の場合は、常にfalse
            //}
            model.setValueAt(new Boolean(tmpEnabled), i, 2);
        }
        repaint();	// 再描画
    }  

    /**
     * 
     * テーブルヘッダーを設定する
     * 
     */
    private void updateColumnHeader(int num) {
    	// 3番目のカラムは無視する
    	if (num == 2) {
    		return;
    	}
    	
        String title;
        if (columnSortValues[num]) {
            title = cols[num] + " [-]";
        } else {
            title = cols[num] + " [+]";
        }

        // 全体を更新する
        updateTableHeaders();

        // 最初のカラム以外は更新する
        JTableHeader header = (JTableHeader) this.getTableHeader();

        if (num > 0) {
            // ヘッダーの更新
            header.getColumnModel().getColumn(num).setHeaderValue(title);
        }

        header.repaint();
    }
    
    /**
     * 
     * テーブルを取得する
     * 
     */
    private SearchResultTable getThisTable() {
    	return this;
    }

    /**
     * 
     * テーブルのヘッダーを更新する
     * 
     */
    public void updateTableHeaders() {
        //ヘッダを取得
        JTableHeader header = (JTableHeader) this.getTableHeader();

        // ヘッダーの更新
        header.getColumnModel().getColumn(0).setHeaderValue(TABLE_HEADER_0);
        header.getColumnModel().getColumn(1).setHeaderValue(TABLE_HEADER_1);
        header.getColumnModel().getColumn(2).setHeaderValue(TABLE_HEADER_2);
        header.getColumnModel().getColumn(3).setHeaderValue(TABLE_HEADER_3);
        header.getColumnModel().getColumn(4).setHeaderValue(TABLE_HEADER_4);
        header.getColumnModel().getColumn(5).setHeaderValue(TABLE_HEADER_5);
        header.getColumnModel().getColumn(6).setHeaderValue(TABLE_HEADER_6);
        header.getColumnModel().getColumn(7).setHeaderValue(TABLE_HEADER_7);
        header.getColumnModel().getColumn(8).setHeaderValue(TABLE_HEADER_8);
        header.getColumnModel().getColumn(9).setHeaderValue(TABLE_HEADER_9);
        
        // 名前の更新
        cols[0] = TABLE_HEADER_0;
        cols[1] = TABLE_HEADER_1;
        cols[2] = TABLE_HEADER_2;
        cols[3] = TABLE_HEADER_3;
        cols[4] = TABLE_HEADER_4;
        cols[5] = TABLE_HEADER_5;
        cols[6] = TABLE_HEADER_6;
        cols[7] = TABLE_HEADER_7;
        cols[7] = TABLE_HEADER_8;
        cols[8] = TABLE_HEADER_9;
    }

}   // end class CorrespondenceTable

//テーブルのデコーダによるソートのクラス
class SortDecorator implements TableModel, TableModelListener {
	private  TableModel realModel;
	private int indexes[];

	public SortDecorator(TableModel model) {
		if(model == null)
			throw new IllegalArgumentException("null models are not allowed");
		this.realModel = model;

		realModel.addTableModelListener(this);
		allocate();
	}
	// 追加メソッド
	public DefaultTableModel getTableModel() {
		return (DefaultTableModel) realModel;
	}
	public Object getValueAt(int row, int column) {
//		System.out.println("ROW="+row+" INDEX="+indexes.length);
		// #kozaki 05/09/08 バグ回避
		if (indexes.length <= row) {
			System.out.println("**ERROR???indexes.length <= row" + "ROW=" + row + " INDEX=" + indexes.length);
			return null;
		}
		// #kozaki 05/06/21 バグ回避のために，実行エラー（ArrayIndexOutOfBoundsException）
		// が出る際の条件分岐を追加
		if (indexes.length == 0) {
			System.out.println("ERROR???indexes.length == 0" + "ROW=" + row + " INDEX=" + indexes.length);
			return realModel.getValueAt(row, column);
		}
		if (realModel.getRowCount() < indexes[row]
				|| realModel.getRowCount() == 0) {
			return null;
		}
		return realModel.getValueAt(indexes[row], column);
	}
	public void setValueAt(Object aValue, int row, int column) {
		realModel.setValueAt(aValue, indexes[row], column);
	}
	public void tableChanged(TableModelEvent e) {
		allocate();
	}
	public void sort(int column) {
		int rowCount = getRowCount();

		for(int i=0; i < rowCount; i++) {
			for(int j = i+1; j < rowCount; j++) {
				if(compare(indexes[i], indexes[j], column) < 0) {
					swap(i,j);
				}
			}
		}
	}
	public void resort(int column) {
		int rowCount = getRowCount();

		for(int i=0; i < rowCount; i++) {
			for(int j = i+1; j < rowCount; j++) {
				if(compare(indexes[i], indexes[j], column) > 0) {
					swap(i,j);
				}
			}
		}
	}
	public void swap(int i, int j) {
		int tmp = indexes[i];
		indexes[i] = indexes[j];
		indexes[j] = tmp;
	}
	public int compare(int i, int j, int column) {
		Object io = realModel.getValueAt(i,column);
		Object jo = realModel.getValueAt(j,column);
		//System.out.println("... compare [" + io + "]-{" + jo + "]");

		int c;
		if (io instanceof Integer && jo instanceof Integer) {	// int型の比較
			if (i > j) {
				c = -1;
			} else if (i < j) {
				c = 1;
			} else {
				c = 0;
			}
		} else {		// その他は文字による比較を行う
			c = jo.toString().compareTo(io.toString());
		}
		return (c < 0) ? -1 : ((c > 0) ? 1 : 0);
	}
	private void allocate() {
		indexes = new int[getRowCount()];

		for(int i=0; i < indexes.length; ++i) {
			indexes[i] = i;
		}
	}
	// TableModel pass-through methods follow ...

	public int getRowCount() {
		return realModel.getRowCount();
	}
	public int getColumnCount() {
		return realModel.getColumnCount();
	}
	public String getColumnName(int columnIndex) {
		return realModel.getColumnName(columnIndex);
	}
	public Class getColumnClass(int columnIndex) {
		return realModel.getColumnClass(columnIndex);
	}
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return realModel.isCellEditable(rowIndex, columnIndex);
	}
	public void addTableModelListener(TableModelListener l) {
		realModel.addTableModelListener(l);
	}
	public void removeTableModelListener(TableModelListener l) {
		realModel.removeTableModelListener(l);
	}
}	// end class SortDecorator