package com.SgwMiniSetup.sample;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

/**
 * スマートゲートウェイ 簡単セットアップGUI(PC版) - 太陽光発電設定ダイアログ
 *
 * @author Copyright (c) 2016 Enegate Co, ltd.
 * @version 1.xx
 */
public class SolarPowerSettingDialog extends JDialog implements ActionListener {
	/**
	 * アプリの定数
	 */
	private static final long serialVersionUID = 1L;

	// ========================================================================================================
	// 各パネルの設定値
	// ========================================================================================================
	private JFrame frame; // 親フレーム情報

	private final static String DIALOG_SOLAR_POWER_TITLE = "太陽光発電設定"; // タイトル
	private final static int DIALOG_SOLAR_POWER_WIDTH = 600; // 幅
	private final static int DIALOG_SOLAR_POWER_HEIGHT = 350; // 高さ

	private final static Color DIALOG_SOLAR_POWER_BG_COLOR = new Color(0, 0, 0); // 背景色(黒系:#000000)
	private final static Color DIALOG_SOLAR_POWER_BORDER_COLOR = new Color(255, 255, 255); // 枠線色(白系:#FFFFFF)
	private final static Color DIALOG_SOLAR_POWER_FONT_COLOR = new Color(255, 255, 255); // 文字色(白系:#FFFFFF)

	// 枠線の太さ
	private final static int DIALOG_SOLAR_POWER_BORDER_WEIGHT_INSIDE = 0; // 内側
	private final static int DIALOG_SOLAR_POWER_BORDER_WEIGHT_OUTSIDE = 2; // 外側

	// 文字の種別
	private final static String DIALOG_SOLAR_POWER_FONT_STYLE_TITLE = Font.SANS_SERIF; // タイトル
	private final static String DIALOG_SOLAR_POWER_FONT_STYLE_MSG1 = Font.SANS_SERIF; // 本文1
	private final static String DIALOG_SOLAR_POWER_FONT_STYLE_MSG2 = Font.SANS_SERIF; // 本文2

	// 文字の太さ
	private final static int DIALOG_SOLAR_POWER_FONT_WEIGHT_TITLE = Font.BOLD; // タイトル
	private final static int DIALOG_SOLAR_POWER_FONT_WEIGHT_MSG1 = Font.BOLD; // 本文1
	private final static int DIALOG_SOLAR_POWER_FONT_WEIGHT_MSG2 = Font.BOLD; // 本文2

	// 文字のサイズ
	private final static int DIALOG_SOLAR_POWER_FONT_SIZE_TITLE = 20; // タイトル
	private final static int DIALOG_SOLAR_POWER_FONT_SIZE_MSG1 = 16; // 本文1
	private final static int DIALOG_SOLAR_POWER_FONT_SIZE_MSG2 = 16; // 本文2

	// ========================================================================================================
	// コンポーネント配置(単位：ピクセル)
	// ========================================================================================================
	// パネル
	private final static int POS_DIALOG_SOLAR_POWER_PANEL_X = 5; // X座標
	private final static int POS_DIALOG_SOLAR_POWER_PANEL_Y = 5; // Y座標
	private final static int POS_DIALOG_SOLAR_POWER_PANEL_WIDTH = 590; // 幅
	private final static int POS_DIALOG_SOLAR_POWER_PANEL_HEIGHT = 340; // 高さ

	// メッセージ1
	private final static String DIALOG_SOLAR_POWER_MSG1 = "<html>太陽光発電を計測している場合は、「あり」を選択の上、<br>" +
			"変更ボタンを押してください。"; // 文言

	private final static String PV_UNAVAILABLE = "なし";
	private final static String PV_AVAILABLE = "あり";
	private final static String NO_SELECTED = "(未設定)";

	private final static int TYPE_PV_UNAVAILABLE = 0;
	private final static int TYPE_PV_AVAILABLE = 1;
	private final static int TYPE_NO_SELECTED = -1;
	private int curPvSetting = TYPE_NO_SELECTED;

	/**
	 * コンストラクタ
	 *
	 * @param JFrame
	 *            frame
	 * @param CardLayout
	 *            layout
	 * @param JPanel
	 *            panelBase
	 * @param String
	 *            success_panel_name
	 * @param String
	 *            error_panel_name
	 */
	public SolarPowerSettingDialog(JFrame frame, String item) {

		// ========================================================================================================
		// 初期化
		// ========================================================================================================
		// 親フレーム継承
		super(frame);
		this.frame = frame;

		setSolarPowerSetting(item);

		// ========================================================================================================
		// 各ダイアログ共通の処理
		// ========================================================================================================
		setTitle(DIALOG_SOLAR_POWER_TITLE);
		setSize(DIALOG_SOLAR_POWER_WIDTH, DIALOG_SOLAR_POWER_HEIGHT); // 幅、高さ
		getContentPane().setBackground(DIALOG_SOLAR_POWER_BG_COLOR); // 背景色
		setLayout(null); // レイアウト無し
		setResizable(false); // フレームのサイズを固定
		setUndecorated(true); // タイトルバー無し
		setLocationRelativeTo(frame); // 親フレームの画面中央に表示
		setModal(true); // モーダル化

		// ========================================================================================================
		// パネル作成。レイアウトは指定しないため、絶対位置で指定する事
		// ========================================================================================================
		JPanel panel = new JPanel();
		panel.setLayout(null);

		// X座標、Y座標、幅、高さをセット
		panel.setBounds(POS_DIALOG_SOLAR_POWER_PANEL_X, POS_DIALOG_SOLAR_POWER_PANEL_Y,
				POS_DIALOG_SOLAR_POWER_PANEL_WIDTH, POS_DIALOG_SOLAR_POWER_PANEL_HEIGHT);

		// 背景色
		panel.setBackground(DIALOG_SOLAR_POWER_BG_COLOR);

		// 外側の枠線の太さ、内側の枠線の太さをセット
		Border outside = BorderFactory.createMatteBorder(
				DIALOG_SOLAR_POWER_BORDER_WEIGHT_OUTSIDE, DIALOG_SOLAR_POWER_BORDER_WEIGHT_OUTSIDE,
				DIALOG_SOLAR_POWER_BORDER_WEIGHT_OUTSIDE, DIALOG_SOLAR_POWER_BORDER_WEIGHT_OUTSIDE,
				DIALOG_SOLAR_POWER_BORDER_COLOR);
		Border inside = BorderFactory.createEmptyBorder(
				DIALOG_SOLAR_POWER_BORDER_WEIGHT_INSIDE, DIALOG_SOLAR_POWER_BORDER_WEIGHT_INSIDE,
				DIALOG_SOLAR_POWER_BORDER_WEIGHT_INSIDE, DIALOG_SOLAR_POWER_BORDER_WEIGHT_INSIDE);
		panel.setBorder(BorderFactory.createCompoundBorder(outside, inside));

		getContentPane().add(panel);

		// ========================================================================================================
		// コンポーネント作成
		// ========================================================================================================

		// タイトル
		JLabel label_title = new JLabel(DIALOG_SOLAR_POWER_TITLE);
		label_title.setBounds(30, 10, 200, 32);
		label_title.setFont(new Font(DIALOG_SOLAR_POWER_FONT_STYLE_TITLE,
				DIALOG_SOLAR_POWER_FONT_WEIGHT_TITLE, DIALOG_SOLAR_POWER_FONT_SIZE_TITLE));
		label_title.setForeground(DIALOG_SOLAR_POWER_FONT_COLOR);
		panel.add(label_title);

		// 区切り線
		JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
		separator.setBounds(5, 50, 580, 15);
		panel.add(separator);

		String selectedItem = NO_SELECTED;
		boolean btnCtrlDisabledEnabled = false;
		boolean btnCtrlEnabledEnabled = false;
		if (curPvSetting == TYPE_PV_UNAVAILABLE) {
			selectedItem = PV_UNAVAILABLE;
			btnCtrlDisabledEnabled = true;
		} else if (curPvSetting == TYPE_PV_AVAILABLE) {
			selectedItem = PV_AVAILABLE;
			btnCtrlEnabledEnabled = true;
		}

		// メッセージ1
		JLabel label_explanation1 = new JLabel(DIALOG_SOLAR_POWER_MSG1);
		label_explanation1.setBounds(30, 60, 530, 80);
		label_explanation1.setFont(new Font(
				DIALOG_SOLAR_POWER_FONT_STYLE_MSG1, DIALOG_SOLAR_POWER_FONT_WEIGHT_MSG1,
				DIALOG_SOLAR_POWER_FONT_SIZE_MSG1));
		label_explanation1.setForeground(DIALOG_SOLAR_POWER_FONT_COLOR);
		panel.add(label_explanation1);

		ButtonGroup group = new ButtonGroup();

		// メッセージ2
		JLabel label_explanation2 = new JLabel("　※ 現在の設定: [ " + selectedItem + " ]");
		label_explanation2.setBounds(90, 200, 400, 80);
		label_explanation2.setFont(new Font(
				DIALOG_SOLAR_POWER_FONT_STYLE_MSG2, DIALOG_SOLAR_POWER_FONT_WEIGHT_MSG2,
				DIALOG_SOLAR_POWER_FONT_SIZE_MSG2));
		label_explanation2.setForeground(DIALOG_SOLAR_POWER_FONT_COLOR);
		panel.add(label_explanation2);

		// オプション１
		JRadioButton radioBtnDisanbled = new JRadioButton(PV_UNAVAILABLE, btnCtrlDisabledEnabled);
		radioBtnDisanbled.setBounds(70, 150, 490, 20);
		radioBtnDisanbled.setFont(new Font(DIALOG_SOLAR_POWER_FONT_STYLE_MSG1, 1, 18));
		radioBtnDisanbled.setForeground(DIALOG_SOLAR_POWER_FONT_COLOR);
		radioBtnDisanbled.setBackground(DIALOG_SOLAR_POWER_BG_COLOR);
		radioBtnDisanbled.addActionListener(this);
		radioBtnDisanbled.setActionCommand("ctrl_disable");
		group.add(radioBtnDisanbled);
		panel.add(radioBtnDisanbled);

		// オプション２
		JRadioButton radioBtnEnabled = new JRadioButton(PV_AVAILABLE, btnCtrlEnabledEnabled);
		radioBtnEnabled.setBounds(70, 190, 490, 20);
		radioBtnEnabled.setFont(new Font(DIALOG_SOLAR_POWER_FONT_STYLE_MSG1, 1, 18));
		radioBtnEnabled.setForeground(DIALOG_SOLAR_POWER_FONT_COLOR);
		radioBtnEnabled.setBackground(DIALOG_SOLAR_POWER_BG_COLOR);
		radioBtnEnabled.addActionListener(this);
		radioBtnEnabled.setActionCommand("ctrl_enable");
		group.add(radioBtnEnabled);
		panel.add(radioBtnEnabled);

		// 変更ボタン
		JButton btn_ok = new JButton("変更");
		btn_ok.setBounds(175, 280, 120, 40);
		btn_ok.setFont(new Font(DIALOG_SOLAR_POWER_FONT_STYLE_MSG1, 1, 16));
		btn_ok.addActionListener(this);
		btn_ok.setActionCommand("Yes");
		panel.add(btn_ok);

		// キャンセルボタン
		JButton btn_cancel = new JButton("キャンセル");
		btn_cancel.setBounds(305, 280, 120, 40);
		btn_cancel.setFont(new Font(DIALOG_SOLAR_POWER_FONT_STYLE_MSG1, 1, 16));
		btn_cancel.addActionListener(this);
		btn_cancel.setActionCommand("No");
		panel.add(btn_cancel);
	}

	/**
	 * アクションのイベント設定
	 *
	 * @param ActionEvent
	 *            e イベント情報
	 */
	public void actionPerformed(ActionEvent e) {
		// コマンド名を取得
		String cmd = e.getActionCommand();

		// 「はい」ボタンを押した場合
		if (cmd.equals("Yes")) {
			// 非表示にする
			setVisible(false);
		}
		// 「いいえ」ボタンを押した場合
		else if (cmd.equals("No")) {
			curPvSetting = TYPE_NO_SELECTED;

			// 非表示にする
			setVisible(false);
		}
		// オプションが選択された場合
		else if (cmd.equals("ctrl_disable")) {
			if (((JRadioButton) e.getSource()).isSelected()) {
				curPvSetting = TYPE_PV_UNAVAILABLE;
			}
		} else if (cmd.equals("ctrl_enable")) {
			if (((JRadioButton) e.getSource()).isSelected()) {
				curPvSetting = TYPE_PV_AVAILABLE;
			}
		}
	}

	public void setSolarPowerSetting(String type) {
		try {
			curPvSetting = Integer.parseInt(type);
		} catch (Exception e) {
			curPvSetting = TYPE_NO_SELECTED;
		}
	}

	public int getSolarPowerSetting() {
		return curPvSetting;
	}
}
