package com.SgwMiniSetup.sample;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.HierarchyBoundsAdapter;
import java.awt.event.HierarchyEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.w3c.dom.Document;

import com.SgwSetting.common.CommonParam;

import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker;
import javafx.embed.swing.JFXPanel;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.control.ScrollPane;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebEvent;
import javafx.scene.web.WebView;

/**
 *
 * SwingのJFrameの上に、JavaFXのWebViewを組み込んだサンプル
 *
 * @author
 */
@SuppressWarnings("serial")
public class WebVeiwOnJFrame extends JFrame {
	private static final String DEFAUTLT_TITLE = "スマートゲートウェイ設定画面表示";

	private JPanel browserPanel;
	
	private WebView webView;
	private WebEngine webEngine;
	
	private int pos_x = 0;
	private int pos_y = 0;
	private int size_w = SgwMiniSetup.WEB_VEW_SIZE_WIDTH;
	private int size_h = SgwMiniSetup.WEB_VEW_SIZE_HEIGHT;
	
	private JFXPanel jFXPanel = new JFXPanel();
	private WebVeiwOnJFrame thisFrame;
	private ScrollPane scrollPane;
	
	private static String url = "http://192.168.0.199/";
		
	private boolean requestAuthentication = false;
	private boolean isAuthenticationRequested = false;
	
	protected boolean displayWebContents = false;
	
	
	public WebVeiwOnJFrame(String url, boolean hasAuthentication, int x, int y, int w, int h) {
		this(url, hasAuthentication, x, y, w, h, false);
	}
	public WebVeiwOnJFrame(String url, boolean hasAuthentication, int x, int y, int w, int h, boolean isWebContents) {
		WebVeiwOnJFrame.url = url;
		this.requestAuthentication = hasAuthentication;
		this.displayWebContents = isWebContents;
		
		if (x > 50) pos_x = x - 50;
		if (y > 50) pos_y = y - 50;
		if (SgwMiniSetup.IS_HIGH_RESOLUTION) {
			if (this.displayWebContents) {
				size_w = SgwMiniSetup.calcDispSize(SgwMiniSetup.WEB_VEW_SIZE_WIDTH_FOR_CONTETNS);
				size_h = SgwMiniSetup.calcDispSize(SgwMiniSetup.WEB_VEW_SIZE_HEIGHT_FOR_CONTETNS);
			} else {
				size_w = SgwMiniSetup.calcDispSize(SgwMiniSetup.WEB_VEW_SIZE_WIDTH);
				size_h = SgwMiniSetup.calcDispSize(SgwMiniSetup.WEB_VEW_SIZE_HEIGHT);
			}
		} else {
			if (this.displayWebContents) {
				size_w = SgwMiniSetup.WEB_VEW_SIZE_WIDTH_FOR_CONTETNS;
				size_h = SgwMiniSetup.WEB_VEW_SIZE_HEIGHT_FOR_CONTETNS;
			} else {
				size_w = SgwMiniSetup.WEB_VEW_SIZE_WIDTH;
				size_h = SgwMiniSetup.WEB_VEW_SIZE_HEIGHT;
			}
		}
		
		// 初期化
		initComponents();

		Platform.runLater(new Runnable() {
			public void run() {
				
				webView = new WebView();
				// サイズの指定
		        webView.setPrefSize(browserPanel.getWidth(), browserPanel.getHeight());
		        if (displayWebContents && SgwMiniSetup.IS_HIGH_RESOLUTION) {
		        	// Web画面を倍率200%（or 300%）にて表示する
		        	webView.setZoom(SgwMiniSetup.DISP_RATE);
		        }
		        
				webEngine = webView.getEngine();
				
				//JavaScriptを有効にする
				webEngine.setJavaScriptEnabled(true);
				
				//コールバック
				webEngine.setOnAlert(new EventHandler<WebEvent<String>>(){
	                public void handle(WebEvent<String> event) {
	                	System.out.println("alert:"+event.getData());
	                }
	            });

				String requestURL = WebVeiwOnJFrame.url;
				if (requestAuthentication && !isAuthenticationRequested) {
					requestURL = "http://" + CommonParam.smartGatewayIp + "/login.cgi";
				}
				webEngine.load(requestURL);
					
				Group group = new Group();
				group.getChildren().add(webView);
				
				// スクロールペイン
				scrollPane = new ScrollPane();
			    scrollPane.setContent(group);
				
				Scene scene = new Scene(group);
				jFXPanel.setScene(scene);
				
				// キーイベント
				jFXPanel.addKeyListener(new KeyAdapter() {
					@Override
		            public void keyPressed(KeyEvent e) { 
						switch(e.getKeyCode()) {
		                    case KeyEvent.VK_F5:
		                    	reloadWebView();	// リロード
		                        break;
		                    default:
		                        // 何もしない
		                }
		            }
		        });

		        Worker<Void> worker = webEngine.getLoadWorker();
		        worker.stateProperty().addListener(new ChangeListener<Worker.State>() {
		            public void changed(ObservableValue<? extends Worker.State> value, Worker.State oldState, Worker.State newState) {
		                if (newState == Worker.State.SUCCEEDED) {
		                    // ページのロードに成功した時の処理
		                	Document doc = webEngine.getDocument();
		        			
		                	if (doc == null || doc.getDocumentElement() == null) {
		                		return;
		                	}
		                	
		        			String location = webEngine.getLocation();
		        			String text = doc.getDocumentElement().getTextContent();
                            //System.out.println(text);
                            
		        			if (requestAuthentication) {
	                            // ログイン画面に限定して処理する
	                            if (location != null && location.indexOf("login.cgi") > 0
	                            		&& text != null && (text.contains("パスワード") || text.contains("User name"))) {
	                            	
	                            	// 属性値に設定
				        			webEngine.executeScript("document.getElementsByName('USER')[0].setAttribute('value','root')");
				        			webEngine.executeScript("document.getElementsByName('PASSWORD')[0].setAttribute('value','root')");
				        			
				        			sleep(200);
		                            
				        			//　Submitする
				        			webEngine.executeScript("document.forms[0].submit()");
				        			System.out.println("[1] " + location);
				        			
	                            } else if (location != null && text != null 
	                            		&& (text.contains("ログインしました。") || text.contains("Logged in."))) {
	                            	// ボタンを強制的にクリック
				        			//engine.executeScript("document.getElementsByName('OK_BTN')[0].click()");
				        			webEngine.executeScript("document.getElementsByTagName('input')[0].click()");
				        			isAuthenticationRequested = true;
				        			System.out.println("[2] " + webEngine.getLocation());
				        			
	                            } else if (location != null && location.indexOf("index.cgi") > 0
	                            		&& text != null && (text.contains("通信設定") || text.contains("Local network"))
	                            		&& isAuthenticationRequested){
	                            	
	                            	webEngine.load(WebVeiwOnJFrame.url);
				        			// リンクを強制的にクリック
				        			//engine.executeScript("document.getElementsByTagName('a')[4].click()");
	                            	
	                            	// 再びリクエストしないようにする
	                            	requestAuthentication = false;
	                            	System.out.println("[3] " + webEngine.getLocation());
	                            	
	                            }
		        			} else {
		        				System.out.println("[0] " + location);
		        			}
		                }
		                
		                // タイトル更新(実行中のみ「読み込み中」表示)
		                thisFrame.updateTitle(newState == Worker.State.RUNNING);
		            }
		        });
			}
		});
	}
	
	public WebVeiwOnJFrame(String url, String title, boolean hasAuthentication, int x, int y, int w, int h, boolean isWebContents) {
		this(url, hasAuthentication, x, y, w, h, isWebContents);
		
		setTitle(title);
	}
	
	protected void updateTitle(boolean isLoading) {
		try {
			if (isLoading) {
				setTitle(DEFAUTLT_TITLE + " - [読み込み中]");
			} else {
				setTitle(DEFAUTLT_TITLE);
			}
        } catch (Exception e) {
        	e.printStackTrace();
        }
	}
	
	private void reloadWebView() {
		Platform.runLater(new Runnable() {
			public void run() {
				try {
					webEngine.reload(); // リロード
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	 /** スリープ */
    static void sleep(long t){
        try {
            Thread.sleep(t);
        } catch (InterruptedException ex) {
        }
    }
    
	private void initComponents() {
		this.thisFrame = this;
		
		// ブラウザペイン
		browserPanel = new JPanel();

		// 閉じるボタンを押すと、アプリが終了する為、コメントアウト
		//setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		// 2回目以降に、WebViewが表示されない為、コメントアウト
		//setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);	// ウインドウを破棄する
		
		setTitle(DEFAUTLT_TITLE);

		setPreferredSize(new Dimension(900, 700));

		//Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		//setBounds((screenSize.width - 900) / 2, (screenSize.height - 700) / 2, 900, 700);
		setBounds(pos_x, pos_y, size_w, size_h);
		browserPanel.setPreferredSize(new Dimension(size_w, size_h));
		
		browserPanel.setLayout(new java.awt.BorderLayout());
		getContentPane().add(browserPanel, BorderLayout.CENTER);
		
		browserPanel.add(jFXPanel, BorderLayout.CENTER);
		
		browserPanel.addHierarchyBoundsListener (
			new HierarchyBoundsAdapter() {
				// 移動したときに呼び出される
				@Override
				public void ancestorMoved(HierarchyEvent e) {
					// System.out.println("ancestor Moved ...");
				}
				// サイズ変更されたときに呼び出される
				@Override
				public void ancestorResized(HierarchyEvent e) {
					//System.out.println("ancestor Resized ...");
					if (thisFrame != null && webView != null) {
						//【＠Ver.2.04】：　リサイズの際のサイズ設定のバグを修正
						if (browserPanel !=null) {
							webView.setPrefSize(browserPanel.getWidth(), browserPanel.getHeight());
						} else {
							webView.setPrefSize(thisFrame.getWidth(), thisFrame.getHeight());
						}
						
						if (SgwMiniSetup.IS_HIGH_RESOLUTION) {
							SgwMiniSetup.WEB_VEW_SIZE_WIDTH = SgwMiniSetup.disCalcDispSize(thisFrame.getWidth());
							SgwMiniSetup.WEB_VEW_SIZE_HEIGHT = SgwMiniSetup.disCalcDispSize(thisFrame.getHeight());
						} else {
							SgwMiniSetup.WEB_VEW_SIZE_WIDTH = thisFrame.getWidth();
							SgwMiniSetup.WEB_VEW_SIZE_HEIGHT = thisFrame.getHeight();
						}
						
						repaintAll();
					}
				}
			}
		);
		
		// アプリのfaviconイメージをセット
		ImageIcon icon_favicon = new ImageIcon(getClass().getClassLoader().getResource(SgwMiniSetup.IMG_APLI_FAVICON));
		setIconImage(icon_favicon.getImage());
		
		// JFrameウインドーを閉じる場合の処理（※コメントアウト --> 理由:一旦閉じると、WebViewが表示されない為）
        //setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}
	
	private void fitWebContentsToFrameSize() {
		// 画面サイズに合せて、Webコンテンツの倍率を指定する
		if (displayWebContents) {
			int curWidth = thisFrame.getWidth();
			int curHeight = thisFrame.getHeight();
			
			if (curWidth > 0 && curHeight > 0) {
				double rate_w = curWidth / SgwMiniSetup.WEB_VEW_SIZE_WIDTH_FOR_CONTETNS;
				double rate_h = curHeight / SgwMiniSetup.WEB_VEW_SIZE_HEIGHT_FOR_CONTETNS;
				double zoom_rate = rate_h;
				if (rate_w < rate_h) { zoom_rate = rate_w; }
				if (zoom_rate > 0) {
					webView.setZoom(zoom_rate);
				}
			}
		}
	}
	
	public void repaintAll() {
		this.repaint();
		
		if (jFXPanel != null
				&& jFXPanel.isVisible()) {
			jFXPanel.updateUI();
		}
		if (browserPanel != null 
				&& browserPanel.isVisible()) {
			browserPanel.updateUI();
		}
	}

	public void launchWebView() {
		this.setVisible(true);
	}
	
	public void setRequestAuthentication(boolean requestAuthentication) {
		this.requestAuthentication = requestAuthentication;
		this.isAuthenticationRequested = !requestAuthentication;
	}
}