package com.SgwMiniSetup.thread;

import java.awt.Color;
import java.io.IOException;
import java.net.InetAddress;

import javax.swing.JOptionPane;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.util.EntityUtils;

import com.SgwMiniSetup.common.CommonFunc;
import com.SgwMiniSetup.sample.SgwMiniSetup;
import com.SgwSetting.common.CommonParam;
import com.SgwSetting.http.SgwHttpClient;
import com.SgwSetting.smartgateway.SearchSgwIpAddress;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

/**
 *
 * 通信テストの送信を行うためのスレッドクラス
 *
 */
public class CommTestThread extends Thread {

	public static final int COM_TEST_TIMEOUT = 5000;//通信テストタイムアウト時間　5秒

	private boolean resultComTest; 		//通信テストの結果
	private String ipAddress; 			//通信テストの結果
	private String ipAddressSub; 		//通信テストの結果
	private boolean needsWarningMessage; //通信テストの結果

	private SgwMiniSetup mainFrame;

	// コンストラクタ
	public CommTestThread(SgwMiniSetup frame, String address, boolean needsWarningMes) {
		mainFrame = frame;
		ipAddress = address;
		ipAddressSub = null;
		needsWarningMessage = needsWarningMes;

		resultComTest = false;//通信テストの結果
	}
	
	// コンストラクタ
	public CommTestThread(SgwMiniSetup frame, String address, String subaddress, boolean needsWarningMes) {
		mainFrame = frame;
		ipAddress = address;
		ipAddressSub = subaddress;
		needsWarningMessage = needsWarningMes;

		resultComTest = false;//通信テストの結果
	}

	public void run() {

		String serialNo = null;
		String resStr = "";

		String checkAddress = ipAddress;
		checkAddress = checkAddress.replace("http://", "");
		checkAddress = checkAddress.replace("/index.cgi", "");
		checkAddress = checkAddress.replace("/index_media.cgi", "");

		CommonParam.checkingCommTest = true;
		
		try {
			// Ping通信チェック
			if (!checkPingConnection(ipAddress)) {
				System.out.println("[警告]: スマートゲートウェイにping接続できませんでした。　<-- " + mainFrame.smartGatewayInfo);
				showMessageDialog(needsWarningMessage);
				// 通信状態を更新する
				updateCommStatus(resultComTest);
				return;
			}
			
			if (CommonParam.useOkHttpClient) {
				
				//【＠Ver.2.04】：Proxy設定のHttpクライアントでTOPページを確認する
				//OkHttpClient okHttpClient = SgwHttpClient.createOkHttpClientWithProxy(2000);
				OkHttpClient okHttpClient = SgwHttpClient.createOkHttpClientWithProxy(CommonParam.HTTP_CONNECTION_TIMEOUT);
				/* ログイン処理 */
				Request request = new Request.Builder()
						.url(ipAddress)
		                .get()
		                .build();
				
				//正常に接続できた場合
				Response response = okHttpClient.newCall(request).execute();
				if (response.isSuccessful()) {	// ステータスコード200
					resultComTest = true;
					
					//【＠Ver.2.04】：SGW-Plusか、エンコード指定がUTF-8の場合のみ、UTF-8でエンコード
					//          それ以外は、デフォールトのShift_JISでエンコード処理する
					if (SgwMiniSetup.IS_SGW_PLUS || SgwMiniSetup.isUtf8Encode()) {
						resStr = CommonFunc.convertInputStreamToString(response.body().byteStream(), "UTF-8");
					} else {
						resStr = CommonFunc.convertInputStreamToString(response.body().byteStream(), "SJIS");
					}
					
					// 製造番号
					serialNo = SearchSgwIpAddress.getSerialNoFromStringText(resStr);
					// エコンコード
					String encode = SearchSgwIpAddress.getEncodeFromStringText(resStr);
					// タイプ
					int type = CommonFunc.getSgwTypeByResString(resStr);
	
					// マップを更新
					SearchSgwIpAddress.smartGatewayInfoMap.put(checkAddress, serialNo);
					SearchSgwIpAddress.smartGatewayEncodeMap.put(checkAddress, encode);
					SearchSgwIpAddress.smartGatewayTypeMap.put(checkAddress, Integer.toString(type));
					
				} else {
					request = new Request.Builder()
			                .url(ipAddressSub)
			                .get()
			                .build();
					response = okHttpClient.newCall(request).execute();
					
					//正常に接続できた場合
					if (response.isSuccessful()) {	// ステータスコード200
						resultComTest = true;
						//SGW-PlusV3の場合のみなので、UTF-8でエンコード
						resStr = CommonFunc.convertInputStreamToString(response.body().byteStream(), "UTF-8");
						
						// 製造番号
						serialNo = SearchSgwIpAddress.getSerialNoFromAuthID(resStr);
						// エコンコード
						String encode = "UTF-8";
						// タイプ
						int type = CommonFunc.getSgwTypeByResString(resStr);
						
						// マップを更新
						SearchSgwIpAddress.smartGatewayInfoMap.put(checkAddress, serialNo);
						SearchSgwIpAddress.smartGatewayEncodeMap.put(checkAddress, encode);
						SearchSgwIpAddress.smartGatewayTypeMap.put(checkAddress, Integer.toString(type));
					}
				}
				
			} else {
				
				//【＠Ver.2.04】：Proxy設定のHttpクライアントでTOPページを確認する
				//DefaultHttpClient httpClient = SgwHttpClient.createHttpClientWithProxy();
				DefaultHttpClient httpClient = SgwHttpClient.createHttpClientWithProxy(CommonParam.HTTP_CONNECTION_TIMEOUT);
				
				HttpParams httpParams = httpClient.getParams();
				//接続確立のタイムアウトを設定（単位：ms）  ⇒ 2秒
				HttpConnectionParams.setConnectionTimeout(httpParams, 2*1000);
				//接続後のタイムアウトを設定（単位：ms）  ⇒ 2秒
				HttpConnectionParams.setSoTimeout(httpParams, 2*1000);
				HttpResponse res = httpClient.execute(new HttpGet(ipAddress));
	
				//正常に接続できた場合
				if (res.getEntity() != null 
						&& res.getStatusLine().getStatusCode() == 200) {	// ステータスコード200
					resultComTest = true;
	
					//【＠Ver.2.04】：SGW-Plusか、エンコード指定がUTF-8の場合のみ、UTF-8でエンコード
					//          それ以外は、デフォールトのShift_JISでエンコード処理する
					if (SgwMiniSetup.IS_SGW_PLUS || SgwMiniSetup.isUtf8Encode()) {
						resStr = EntityUtils.toString(res.getEntity(), "UTF-8");
					} else {
						resStr = EntityUtils.toString(res.getEntity(), "SJIS");
					}
	
					// エンティティの後処理（エンティティ内のInputStreamのクローズ処理)
					EntityUtils.consume(res.getEntity());
	
					// 製造番号
					serialNo = SearchSgwIpAddress.getSerialNoFromStringText(resStr);
					// エコンコード
					String encode = SearchSgwIpAddress.getEncodeFromStringText(resStr);
					// タイプ
					int type = CommonFunc.getSgwTypeByResString(resStr);
	
					// マップを更新
					SearchSgwIpAddress.smartGatewayInfoMap.put(checkAddress, serialNo);
					SearchSgwIpAddress.smartGatewayEncodeMap.put(checkAddress, encode);
					SearchSgwIpAddress.smartGatewayTypeMap.put(checkAddress, Integer.toString(type));
				}
				else {
					res = httpClient.execute(new HttpGet(ipAddressSub));
					
					//正常に接続できた場合
					if (res.getEntity() != null 
							&& res.getStatusLine().getStatusCode() == 200) {	// ステータスコード200
						resultComTest = true;
						
						//SGW-PlusV3の場合のみなので、UTF-8でエンコード
						//resStr = EntityUtils.toString(res.getEntity(), "UTF-8");
						resStr = new String(EntityUtils.toString(res.getEntity()).getBytes("ISO8859-1"), "UTF-8");
						
						// 製造番号
						serialNo = SearchSgwIpAddress.getSerialNoFromAuthID(resStr);
						// エコンコード
						String encode = "UTF-8";
						// タイプ
						int type = CommonFunc.getSgwTypeByResString(resStr);
						
						// マップを更新
						SearchSgwIpAddress.smartGatewayInfoMap.put(checkAddress, serialNo);
						SearchSgwIpAddress.smartGatewayEncodeMap.put(checkAddress, encode);
						SearchSgwIpAddress.smartGatewayTypeMap.put(checkAddress, Integer.toString(type));
					}
				}
			}
			
		} catch (IOException ie) {
			//ie.printStackTrace();
			System.out.println("[警告]: スマートゲートウェイにhttp接続できませんでした。　<-- " + mainFrame.smartGatewayInfo);
			//メッセージ表示
			showMessageDialog(needsWarningMessage);
		} catch (Exception e) {
			e.printStackTrace();
			//throw e;
		} finally {
			CommonParam.checkingCommTest = false;
		}

		// 取得した製造番号を一時置き
		if (serialNo != null && serialNo.length() == 8) {
			CommonParam.checkedSerialNo = serialNo;
		}

		// SGWタイプを判定する
		int type = CommonFunc.getSgwTypeByResString(resStr);
		// マップに格納する
		SearchSgwIpAddress.smartGatewayTypeMap.put(checkAddress, Integer.toString(type));
		// SGWタイプを設定する
		SgwMiniSetup.setSmartGatewayType(type);

		String status = "";
		if (type == SgwMiniSetup.TYPE_SGW_PLUS_MC) {
			status = "MediaConv";
		} else {
			//【＠Ver.2.04】： SGWの登録状態を取得する
			status = CommonFunc.getSgwRegistrationStatus(resStr);
		}
		//【＠Ver.2.04】： SGWの登録状態を設定する
		SgwMiniSetup.setSmartGatewayStatus(status);

		// 通信状態を更新する
		updateCommStatus(resultComTest);

		// 認証チェック
		if (SgwMiniSetup.IS_SGW_PLUS && !SgwMiniSetup.IS_SGW_PLUS_MC) {
			CheckAuthorizationThread th = new CheckAuthorizationThread(mainFrame, checkAddress);
			th.start();
		}
	}

	private void showMessageDialog(boolean needsWarningMessage) {
		if (needsWarningMessage) {
			String info = mainFrame.smartGatewayInfo;
			if (info != null && info.contains(CommonParam.REGIST_STATUS_LBL)) {
				int index = info.indexOf(CommonParam.REGIST_STATUS_LBL);
				info = info.substring(0, index);
			}
			String mes = "前回接続した、以下のスマートゲートウェイに接続できませんでした。\n\n"
							+ info + "　　　　\n\n";
			String title = "スマートゲートウェイに接続できませんでした";
			JOptionPane.showMessageDialog(mainFrame, mes, title, JOptionPane.INFORMATION_MESSAGE);
		}
	}

	private boolean checkPingConnection(String httpAddress) {
		try {
			String checkAddress = httpAddress;
			checkAddress = checkAddress.replace("http://", "");
			checkAddress = checkAddress.replace("/index.cgi", "");
			checkAddress = checkAddress.replace("/index_media.cgi", "");
			InetAddress inetAddress = InetAddress.getByName(checkAddress);
			boolean isMacOS = SearchSgwIpAddress.isMacOS();
			boolean isLinuxOS = SearchSgwIpAddress.isLinuxOS();
			boolean resIcmp = false;
			String[] command = { "ping", "-n", "2", "-w", "30", inetAddress.getHostAddress() };
			String[] command_for_mac = { "ping", "-c", "2", inetAddress.getHostAddress() };
			if (isMacOS || isLinuxOS) {
				resIcmp = new ProcessBuilder(command_for_mac).start().waitFor() == 0;
			} else {
				resIcmp = new ProcessBuilder(command).start().waitFor() == 0;
			}
			return resIcmp;
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return false;
	}

	private void updateCommStatus(boolean isSucceeded) {
		mainFrame.isSGWInfoChecked = isSucceeded;

		if (mainFrame != null
				&& mainFrame.label_footer_sgw_info != null) {
			// 表示色を変更にする
			if (isSucceeded) {
				mainFrame.label_footer_sgw_info.setForeground(Color.BLACK);
			} else {
				mainFrame.label_footer_sgw_info.setForeground(Color.MAGENTA);
			}
		}

		// スマートゲートウェイの情報を更新
		// （※特に初回時に「（不明）」となっているので、上書き更新する必要がある）
		mainFrame.updateSgwInfo();

		// ボタンの表示/非表示
		SgwMiniSetup.setSmartGatewayType(CommonParam.smartGatewayType);
		mainFrame.setButtonVisiblity(isSucceeded);
	}

	public boolean getResult() {
		return resultComTest;
	}
}
