package com.SgwMiniSetup.thread;

import java.awt.Toolkit;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JFrame;

import com.SgwSetting.common.CommonParam;
import com.SgwSetting.smartgateway.NetworkInfo;
import com.SgwSetting.smartgateway.SearchSgwIpAddress;
import com.SgwSetting.thread.ThreadStart;


/**
 * スマートゲートウェイの検索スレッド
 */
public class SgwSearchThread extends Thread {
	/**
	 * クラス変数
	 */
	public String sgw_ip;     // 見つかったSGWのIPアドレス
	public boolean status;    // SGWのIPアドレスが見つかったか(true:見つかった、false:見つからなかった)

	private long start = 0;
	private long end = 0;
	
	private JFrame frame = null;
	
	private boolean isActive = true;

	/**
	 * コンストラクタ
	 */
	public SgwSearchThread() {
		// 初期化
		this.sgw_ip = "";
		this.status = false;
	}
	
	public void setFrame(JFrame frame) {
		this.frame = frame;
	}


	/**
	 * スレッド開始
	 */
	public void run() {
		
    	CommonParam.searchEndFlg = false;
    	CommonParam.findSgwForSearching = false;
		start = System.currentTimeMillis();
		
		SearchSgwIpAddress adress = new SearchSgwIpAddress();
		ArrayList<NetworkInfo> networkInfoList = adress.getAnotherIpAddress();
		
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		//IPアドレスを複数持っているかのどうかで分岐する
		if (networkInfoList != null && networkInfoList.size() > 1) {
			/*IPアドレスの数だけループさせる*/
			for (NetworkInfo networkInfo : networkInfoList) {
				if (!this.isActive) {
					break;
				}
				
				//commonParamの各種ネットワーク情報を書き換える
				CommonParam.subnetAddress = networkInfo.subnetAdress;
				CommonParam.networkAddress = networkInfo.networkAddress;
				CommonParam.broadCastAddress = networkInfo.broadCastAddress;
				
				// スマートゲートウェイのIPアドレスを検索
				ThreadStart th    = new ThreadStart();
				//this.sgw_ip       = th.StartThreadSearchSgw();
				this.sgw_ip       = th.StartThreadSearchSgw(this.frame);
				
				String[] checkstr = this.sgw_ip.split("\\.");
				// SGWのIPアドレスが見つかったか
				if(checkstr.length == 4) {
					// 検索に成功
					this.status = true;
				}
				
				// SGWの検索範囲を減算
				CommonParam.restThread = CommonParam.THREAD_NUM_SGW_SEARCH;
			}
		} else {
			adress.calcNetworkAddress();
			// スマートゲートウェイのIPアドレスを検索
			ThreadStart th    = new ThreadStart();
			this.sgw_ip       = th.StartThreadSearchSgw();
			
			String[] checkstr = this.sgw_ip.split("\\.");
			// SGWのIPアドレスが見つかったか
			if(checkstr.length == 4) {
				// 検索に成功
				this.status = true;
			}
			
			// SGWの検索範囲を減算
			CommonParam.restThread = CommonParam.THREAD_NUM_SGW_SEARCH;
		}
		
		// 終了直前の動作
		for(int i = 0; i < 5; i++) {
			toolkit.beep();
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * スレッドを停止します。
	 */
	public void stopThread() {
		this.isActive = false;
	}

	/**
	 * スマートゲートウェイのIPアドレスが見つかったかどうかを返す
	 *
	 * @return boolean true:見つかった、false:見つからなかった
	 */
	public boolean getSgwStatus() {
		return this.status;
	}


	/**
	 * 見つかったスマートゲートウェイのIPアドレスを返す
	 *
	 * @return String 見つかったIPアドレス
	 */
	public String getSgwIp() {
		return this.sgw_ip;
	}
}
