package com.SgwSetting.http;

public class HTMLAnalyzer {

	private static String VALUE_START = "value=\"";
	private static String VALUE_END = "\"";

	private static String OPTION_SELECTED_1 = " selected=\"selected\">";
	private static String OPTION_SELECTED_2 = " selected >";
	private static String OPTION_END = "</option>";

	private static String OPTION_CHECKED = "checked";

	/**
	 * 文字列から指定した開始タグと終了タグの間の値を取得する
	 *
	 * @param text 文字列を指定
	 * @param position 文字列の開始位置を指定
	 * @param startTag 開始タグを指定
	 * @param endTag 終了タグを指定
	 */
	public static String getTagElementValue(String text, int position, String startTag, String endTag) {
		String rtnText = null;
		int startIndex = text.indexOf(startTag, position);
		if (startIndex >= 0 && startIndex < text.length()) {
			int startValueIndex = startIndex + startTag.length();
			int endValueIndex = text.indexOf(endTag, startValueIndex);
			if (endValueIndex > 0 && endValueIndex < text.length()) {
				rtnText = text.substring(startValueIndex, endValueIndex);
			}
		}
		return rtnText;
	}

	/**
	 * 文字列から指定したタグのvalue属性の値を取得する
	 *
	 * @param text 文字列を指定
	 * @param position 文字列の開始位置を指定
	 * @param startTag 開始タグを指定
	 */
	public static String getTagAttributeValue(String text, int position, String startTag) {
		String rtnText = null;
		int startIndex = text.indexOf(startTag, position);
		if (startIndex >= 0 && startIndex < text.length()) {
			int valueStartIndex = text.indexOf(VALUE_START, startIndex);
			int startValueIndex = valueStartIndex + VALUE_START.length();
			int endValueIndex = text.indexOf(VALUE_END, startValueIndex);
			if (endValueIndex > 0 && endValueIndex < text.length()) {
				rtnText = text.substring(startValueIndex, endValueIndex);
			}
		}
		return rtnText;
	}

	/**
	 * 文字列から指定したSelectタグの選択オプションの値を取得する
	 *
	 * @param text 文字列を指定
	 * @param position 文字列の開始位置を指定
	 * @param startTag 開始タグを指定
	 * @param endTag 終了タグを指定
	 */
	public static String getSelectTagOptionValue(String text, int position, String startTag, String endTag) {
		String rtnText = null;
		int startIndex = text.indexOf(startTag, position);
		if (startIndex >= 0 && startIndex < text.length()) {
			String strOption = OPTION_SELECTED_1;
			int selectedStartIndex = text.indexOf(strOption, startIndex);
			if (selectedStartIndex < 0) {
				strOption = OPTION_SELECTED_2;
				selectedStartIndex = text.indexOf(strOption, startIndex);
			}
			if (selectedStartIndex >= 0) {
				int startValueIndex = selectedStartIndex + strOption.length();
				int endValueIndex = text.indexOf(OPTION_END, startValueIndex);
				if (endValueIndex > 0 && endValueIndex < text.length()) {
					rtnText = text.substring(startValueIndex, endValueIndex);
				}
			}
		}
		return rtnText;
	}

	/**
	 * 文字列から指定したRadioボタンの選択オプションの値を取得する
	 *
	 * @param text 文字列を指定
	 * @param position 文字列の開始位置を指定
	 * @param endTag 終了タグを指定
	 */
	public static String getRadioTagOptionValue(String text, int position, String startTag, String endTag) {
		String rtnText = null;
		int startIndex = text.indexOf(startTag, position);
		int endIndex = -1;
		if (startIndex >= 0 && startIndex < text.length()) {
			endIndex = text.indexOf(endTag, startIndex);
			int radioStartIndex = text.indexOf(OPTION_CHECKED, startIndex);
			if (radioStartIndex < 0) {
				return null;
			} else if (radioStartIndex < endIndex) {
				// 何もしない
			} else if (radioStartIndex > endIndex) {
				// 次のタグを探す
				return getRadioTagOptionValue(text, endIndex, startTag, endTag);
			}
			if (startIndex >= 0 && radioStartIndex >= 0) {
				int valueStartIndex = text.indexOf(VALUE_START, startIndex);
				int startValueIndex = valueStartIndex + VALUE_START.length();
				int endValueIndex = text.indexOf(VALUE_END, startValueIndex);
				if (endValueIndex > 0 && endValueIndex < text.length()) {
					rtnText = text.substring(startValueIndex, endValueIndex);
				}
			}
		}
		return rtnText;
	}
}
