package com.SgwSetting.http;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.ProxySelector;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.concurrent.TimeUnit;

import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.PoolingClientConnectionManager;
import org.apache.http.impl.conn.ProxySelectorRoutePlanner;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.NameValuePair;

import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.params.CookiePolicy;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.ProxySelectorRoutePlanner;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.riversun.okhttp3.OkHttp3CookieHelper;

import com.SgwSetting.common.CommonParam;
import com.SgwSetting.http.LowSecuritySSLSocketFactory;

import okhttp3.OkHttpClient;
import okhttp3.OkHttpClient.Builder;

public class SgwHttpClient {
	/* {src_lang=Java}*/
	public static HttpURLConnection uCon;

	/**
	 * オブジェクト作成と同時にソケット作成
	 *
	 * @param url リクエストを行うURL
	 */
	public SgwHttpClient (String url)
	 throws MalformedURLException, ProtocolException, IOException {
		// URL オブジェクトの生成
		URL urlObject = new URL(url);
		// HttpURLConnection オブジェクトの生成
		uCon = (HttpURLConnection)urlObject.openConnection();
		uCon.setReadTimeout(1200);//getResponseCode()に対してレスポンスを返さないサーバも存在するためタイマーを設定
		uCon.setConnectTimeout(1200);//HTTP接続タイマー
	}

	/**
	 * GET リクエストを発行
	 */
	public void doGet ()
	 throws ProtocolException, IOException {
		// リクエストメソッド
		uCon.setRequestMethod("GET");
		// 接続・リクエスト発行
		uCon.connect();
	}

	/**
	 * POST リクエストを発行
	 *
	 * @param message リクエストメッセージ
	 */
	public void doPost (String message)
	 throws ProtocolException, IOException {
		// リクエストメソッド
		uCon.setRequestMethod("POST");
		// メッセージがある場合
		setRequestMessage(message);
		// 接続・リクエスト発行
		uCon.connect();
	}

	/**
	 * リクエストメッセージを設定
	 *
	 * @param message リクエストメッセージ
	 */
	void setRequestMessage (String message) throws IOException {
		// リクエストボディを投げられるようにする
		uCon.setDoOutput(true);
		// リクエストボディの MIME は application/x-www-form-urlencoded
		uCon.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		// リクエストボディの長さ
		uCon.setRequestProperty("Content-Length", Integer.toString(message.length()));
		// リクエストメッセージを流し込む出力ストリーム
		BufferedWriter outer = new BufferedWriter(new OutputStreamWriter(uCon.getOutputStream()));
		outer.write(message);
		outer.flush();
		outer.close();
	}

	/**
	 * ステータスコードの取得
	 *
	 * @return 3桁のステータスコード
	 */
	public int getStatusCode () throws IOException {
		return uCon.getResponseCode();
	}

	/**
	 * レスポンスを受け取るためのストリームの取得
	 *
	 * @return レスポンスを受け取るためのストリーム
	 */
	public InputStream getResponseStream () throws IOException {
		return uCon.getInputStream();
	}

	/**
	 * レスポンスボディを文字列として得る
	 *
	 * @return 文字列としてのレスポンスボディ
	 */
	public String getResponseToString () throws IOException {
		// レスポンスボディを受け取るための入力ストリーム
		BufferedReader inner = new BufferedReader(new InputStreamReader(getResponseStream()));
		String response = "";
		String line = "";
		while ((line = inner.readLine()) != null) {
			response += line + "\n";
		}
		return response;
	}

	public void close() {
		uCon.disconnect();
	}
	
	/**
	 * Proxyを設定したHttpClientを生成する
	 *
	 * @return 生成したHttpClientを戻す。
	 */
	public static DefaultHttpClient createHttpClientWithProxy(int timeout) {
		return createHttpClientWithProxy(timeout, CommonParam.isLowSecurityMode);
	}
	public static DefaultHttpClient createHttpClientWithProxy(int timeout, boolean isLowSecurity) {
		//DefaultHttpClient httpClient = new DefaultHttpClient();	//for文のループの外側に移動しました
		//DefaultHttpClient httpClient = new SystemDefaultHttpClient();
		ClientConnectionManager connManager = new PoolingClientConnectionManager();
		DefaultHttpClient httpClient = new DefaultHttpClient(connManager);
		
		if (isLowSecurity) {
			KeyStore trustStore = null;
			SSLSocketFactory sf = null;
			try {
				trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
				trustStore.load(null, null);
	
				sf = new LowSecuritySSLSocketFactory(trustStore);
				sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
			} catch (Exception e) {
				e.printStackTrace();
			} 
	
			if (trustStore != null && sf != null) {
				HttpParams params = new BasicHttpParams();
				HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
				HttpProtocolParams.setContentCharset(params, HTTP.UTF_8);
				//接続確立のタイムアウトを設定（単位：ms）
				HttpConnectionParams.setConnectionTimeout(params, timeout);
	            //接続後のタイムアウトを設定（単位：ms）
				HttpConnectionParams.setSoTimeout(params, timeout);
		
				SchemeRegistry registry = new SchemeRegistry();
				registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
				registry.register(new Scheme("https", sf, 443));
		
				ClientConnectionManager ccm = new ThreadSafeClientConnManager(params, registry);
				
				httpClient = new DefaultHttpClient(ccm, params);
			} else {
				httpClient = new DefaultHttpClient();
			}
		} 
		else {
			httpClient = new DefaultHttpClient();
		}
		
		// デフォルトのProxySelectorを利用するよう設定
		ProxySelectorRoutePlanner routePlanner = new ProxySelectorRoutePlanner(
				httpClient.getConnectionManager().getSchemeRegistry(), ProxySelector.getDefault());
		httpClient.setRoutePlanner(routePlanner);
		
		return httpClient;
	}
	
	/**
	 * Proxyを設定したOkHttpClientを生成する
	 *
	 * @return 生成したHttpClientを戻す。
	 */
	public static OkHttpClient createOkHttpClientWithProxy(int timeout) {
		return createOkHttpClientWithProxy(CommonParam.isLowSecurityMode, timeout);
	}
	@SuppressWarnings("deprecation")
	private static OkHttpClient createOkHttpClientWithProxy(boolean isLowSecurity, int timeout) {
//		ProxyConfig proxyConfig = new ProxyConfig();
//		int proxyPort = proxyConfig.getPort();
//		String proxyHost = proxyConfig.getHost();
//		
//		final String username = "username";
//		final String password = "password";
//
//		Authenticator proxyAuthenticator = new Authenticator() {
//		  @Override public Request authenticate(Route route, Response response) throws IOException {
//		       String credential = Credentials.basic(username, password);
//		       return response.request().newBuilder()
//		           .header("Proxy-Authorization", credential)
//		           .build();
//		  }
//		};
		
		// OkHttpClientを生成
		OkHttpClient client;
		Builder builder = new OkHttpClient.Builder();
		builder.connectTimeout(timeout, TimeUnit.MILLISECONDS);
		builder.writeTimeout(timeout, TimeUnit.MILLISECONDS);
		builder.readTimeout(timeout, TimeUnit.MILLISECONDS);
//		if (proxyHost != null && proxyPort > 0) {
//			builder.proxy(new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyHost, proxyPort)));
////	    	builder.proxyAuthenticator(proxyAuthenticator);
//		}
		// Set proxy selector
		builder.proxySelector(ProxySelector.getDefault());

		// Android端末が古い（Android4.4以下）場合には、SSL認証局の検証（チェック）を行わないようにする
		if (isLowSecurity) {
			// 【参考URL】 OkHttpで認証・SSL通信・クッキー管理するTips
			// https://qiita.com/muran001/items/ddbd0341670fb03dce05
			builder.sslSocketFactory(trustAllSslSocketFactory, (X509TrustManager)trustAllCerts[0]);
			builder.hostnameVerifier(new HostnameVerifier() {
		        public boolean verify(String hostname, SSLSession session) {
		        	return true;
		        }
		    });
		}
		
		// 【参考】 OkHttp3 で クライアント側から setCookie する
		// https://qiita.com/riversun/items/0855a3af29215aa3c6e3
		OkHttp3CookieHelper cookieHelper = new OkHttp3CookieHelper();
		builder.cookieJar(cookieHelper.cookieJar());
		
		client = builder.build();
		
		return client;
	}
	
	/**
	 * This is very bad practice and should NOT be used in production.
	 */
	private static final TrustManager[] trustAllCerts = new TrustManager[] {
	    new X509TrustManager() {
	        public void checkClientTrusted(java.security.cert.X509Certificate[] chain, String authType) throws CertificateException {
	        }

	        public void checkServerTrusted(java.security.cert.X509Certificate[] chain, String authType) throws CertificateException {
	        }

	        public java.security.cert.X509Certificate[] getAcceptedIssuers() {
	          return new java.security.cert.X509Certificate[]{};
	        }
	    }
	};
	private static final SSLContext trustAllSslContext;
	static {
	    try {
	        trustAllSslContext = SSLContext.getInstance("SSL");
	        trustAllSslContext.init(null, trustAllCerts, new java.security.SecureRandom());
	    } catch (NoSuchAlgorithmException ne) {
	        throw new RuntimeException(ne);
	    } catch (KeyManagementException ke) {
	        throw new RuntimeException(ke);
	    }
	}
	private static final javax.net.ssl.SSLSocketFactory trustAllSslSocketFactory = trustAllSslContext.getSocketFactory();
}