package com.SgwSetting.thread;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;

import com.SgwSetting.common.CommonParam;
import com.SgwSetting.smartgateway.SearchSgwIpAddress;

public class SendMulticastToSgw extends Thread {
	
	public static final int ECHO_PORT = 3610;
	//public static final int PACKET_SIZE = 1024;
	public static final int PACKET_SIZE = 16;
	public static final String MCAST_ADDRESS = "224.0.23.0";
	public static final String ENCODING = "US-ASCII";
	
	/* ECHONET機器取得通知メッセージ電文 */
	public static final String DEFAULT_MESSAGE_HEX = "1081000105FF010EF0016201D600";
	
	private String hostAddress = null;
	private boolean isRecieving = false;

	public SendMulticastToSgw() {
		try {
	    	InetAddress myHost = InetAddress.getLocalHost();
	    	this.hostAddress = myHost.getHostAddress();
		} catch (Exception e) {}
	}
	public SendMulticastToSgw(String inetAddress) {
		this.hostAddress = inetAddress;
	}
	
	public void run() {
		// マルチキャスト受信を起動
		startRecievingMulticast();
	}
	
	public void startRecievingMulticast() {
		MulticastSocket socket = null;
		byte[] buf = new byte[PACKET_SIZE];
		DatagramPacket packet = new DatagramPacket(buf, buf.length);

		try {
			socket = new MulticastSocket(ECHO_PORT);
			InetAddress mcastAddress = InetAddress.getByName(MCAST_ADDRESS);
			socket.joinGroup(mcastAddress);
			if (this.hostAddress != null && this.hostAddress.length() > 0) {
				InetAddress ipaddr = InetAddress.getByName(this.hostAddress);
				socket.setInterface(ipaddr);
			}
			System.out.println("MulticastReceiverを起動しました(" + "host=" + this.hostAddress + ", address=" + mcastAddress + ", port=" + socket.getLocalPort() + ")");
			
			isRecieving = true;
			while (isRecieving) {
				socket.receive(packet);
				
				String message = "";
				for (int i=0;i<buf.length;++i) {
					message += String.format("%02X ", buf[i]);
				}
				
				InetAddress inetAddress = packet.getAddress();
				String packetAddress = packet.getSocketAddress().toString(); 
				System.out.println("Multicastを受信しました(" + "address=" + packetAddress + " : " + message + ")");
				
				// SGWかどうかを確認する
				if (packetAddress != null && packetAddress.indexOf(this.hostAddress) < 0) {
					boolean hasSmartGatewayInfo = SearchSgwIpAddress.checkSmartGatewayInfo(inetAddress);
					if (hasSmartGatewayInfo) {
						CommonParam.findSgwForSearching = true;
					}
				}
			}
		} catch (IOException ie) {
			ie.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (socket != null) {
				socket.close();
			}
		}
	}
	
	public void stopRecievingMulticast() {
		isRecieving = false;
	}
	
	public void sendMulticast() {
		MulticastSocket socket = null;

		try {
			InetAddress mcastAddress = InetAddress.getByName(MCAST_ADDRESS);
			BufferedReader keyIn = new BufferedReader(new InputStreamReader(System.in));
			socket = new MulticastSocket();

			String message = DEFAULT_MESSAGE_HEX;
//			byte[] bytes = message.getBytes(ENCODING);
//			try {
//				bytes = Hex.decodeHex(message);
//			} catch (DecoderException e) {
//				e.printStackTrace();
//			}
			byte[] bytes = hex2bin(message); 
			
			DatagramPacket packet = new DatagramPacket(bytes, bytes.length, mcastAddress, ECHO_PORT);
			socket.send(packet);
			
			String send_message = "";
			for (int i=0;i<bytes.length;++i) {
				send_message += String.format("%02X ", bytes[i]);
			}
			
			System.out.println("Multicastを送信しました(" + "address=" + packet.getSocketAddress() + " : " + send_message + ")");
			
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (socket != null) {
				socket.close();
			}
		}
	}
	
	public static byte[] hex2bin(String hex) {
	    byte[] bytes = new byte[hex.length() / 2];
	    for (int index = 0; index < bytes.length; index++) {
	        bytes[index] = (byte) Integer.parseInt(hex.substring(index * 2, (index + 1) * 2), 16);
	    }
	    return bytes;
	}
}
